#include "suff_0_4debug.h"

/* ----------------------------------------------------------
	+++	GLOBAL DATA USED BY MANY FUNCTIONS +++ 
   ---------------------------------------------------------- */

/* stores information about the current situation on the chess board */
static situation* CurrentBoard = NULL;

static short REAL_DEPTH = 0;
static short REAL_BUCKET_DEPTH = 0;

/* here we will store previous positions, which should help to recognize
   repetitions */
static situation* Reference = NULL;

/* this crazy 4-dim Array stores all possible moves of an empty chess board */
static Destination**** fixed_Destination = NULL;

/* these are used to evaluate possible moves */
static situation* Try_Board = NULL;

static move GlobalNullMove = {'\0','\0','\0','\0', '\0', 0};

static move_table* Try_Table = NULL;

/* this is used to store possible moves */
static move_table* Possible_Moves = NULL;

static move_table* eval_table =	NULL;

static move_table* pre_sort = NULL;	/* NOT USED ANY LONGER ! */

/* +++ BUCKET NEW +++ */
/* These two numbers will be necessary to implement a 
   horizon penalty, I know global loop controllers are ugly ... */
static int buck_i[MAX_DEPTH+2] = {0};
static int buck_j[MAX_DEPTH+2] = {0};
static bucket_criteria* depth_pre_sort = NULL;
/* +++ BUCKET +++ */

static unsigned whiteNum = 0, blackNum = 0;

static move_table* hyper_cont = NULL;

static short curr_depth = 0; /* this_is_used to determine curr_depth */

/* Contains the Fenstring of the starting position */
char* StartPosition = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";

FILE* logFile = NULL; /* I used this for debugging. */







/* -------------------------------------------------
			PROJECT: Variation-Tree
   ------------------------------------------------- */
#define VAR_CONT_SIZE 16384

struct variation{
	move made[3];
	short quality;
};

struct node{
	variation key;
	node* smaller, *bigger;
};

static size_t numOfNodes_left = 0, numOfNodes_right = 0;
static node* NodeCont_left = NULL, *NodeCont_right = NULL;
static node* ROOT_left = NULL, *ROOT_right = NULL;

void insert_node_left(node** curNode, variation* extraVar);
void insert_node_right(node** curNode, variation* extraVar);
node* find_node_left(node* curNode, variation* sKey);
node* find_node_right(node* curNode, variation* sKey);

int init_tree(){
	NodeCont_left = (node*) calloc(VAR_CONT_SIZE,sizeof(node));
	if(!NodeCont_left)
		return FALSE;
	NodeCont_right = (node*) calloc(VAR_CONT_SIZE,sizeof(node));
	return NodeCont_right ? TRUE : FALSE;
}

void reset_tree(){
	ROOT_left = ROOT_right = NULL;
	numOfNodes_left = numOfNodes_right = 0;
}

void insert(variation* extraVar){
	if(GET_FILE(extraVar->made[0].start) <= 3)
		insert_node_left(&ROOT_left, extraVar);
	else
		insert_node_right(&ROOT_right, extraVar);
}

#define NOT_FOUND 32001
short get_quality(variation* extraVar){
	node* to_find = NULL;
	
	if(GET_FILE(extraVar->made[0].start) <= 3)
		to_find = find_node_left(ROOT_left, extraVar);
	else
		to_find = find_node_right(ROOT_right, extraVar);

	if(to_find != NULL)
		return to_find->key.quality;
	return NOT_FOUND;
}

void insert_node_left(node** curNode, variation* extraVar){
	if(!(*curNode)){
		if(numOfNodes_left < VAR_CONT_SIZE){
			memcpy(&(NodeCont_left[numOfNodes_left].key), extraVar, sizeof(variation));
			NodeCont_left[numOfNodes_left].smaller = NULL;
			NodeCont_left[numOfNodes_left].bigger = NULL;
			(*curNode) = NodeCont_left+numOfNodes_left;
			++numOfNodes_left;
		}
	}
	else{
		if(memcmp(extraVar->made, (*curNode)->key.made, sizeof(move)*3) < 0)
			insert_node_left(&((*curNode)->smaller), extraVar);			
		else
			insert_node_left(&((*curNode)->bigger), extraVar);
	}
}

node* find_node_left(node* curNode, variation* sKey){
	int compare = 0;
	if(curNode == NULL)
		return NULL;
	compare = memcmp(sKey->made+0,curNode->key.made+0,sizeof(move)*3);
	if(!compare)
		return curNode;
	else if(compare < 0)
		return find_node_left(curNode->smaller, sKey);
	else if(compare > 0)
		return find_node_left(curNode->bigger, sKey);
	return NULL;	
}

void insert_node_right(node** curNode, variation* extraVar){
	if(!(*curNode)){
		if(numOfNodes_right < VAR_CONT_SIZE){
			memcpy(&(NodeCont_right[numOfNodes_right].key), extraVar, sizeof(variation));
			NodeCont_right[numOfNodes_right].smaller = NULL;
			NodeCont_right[numOfNodes_right].bigger = NULL;
			(*curNode) = NodeCont_right+numOfNodes_right;
			++numOfNodes_right;
		}
	}
	else{
		if(memcmp(extraVar->made, (*curNode)->key.made, sizeof(move)*3) < 0)
			insert_node_right(&((*curNode)->smaller), extraVar);			
		else
			insert_node_right(&((*curNode)->bigger), extraVar);
	}
}

node* find_node_right(node* curNode, variation* sKey){
	int compare = 0;
	if(curNode == NULL)
		return NULL;
	compare = memcmp(sKey->made+0,curNode->key.made+0,sizeof(move)*3);
	if(!compare)
		return curNode;
	else if(compare < 0)
		return find_node_right(curNode->smaller, sKey);
	else if(compare > 0)
		return find_node_right(curNode->bigger, sKey);
	return NULL;	
}
/* -------------------------------------------------
		 END OF PROJECT: Variation-Tree
   ------------------------------------------------- */




/* ----------------------------------------------
		EVALUATION FUNCTIONS + DATA
   ---------------------------------------------- */
const iSquare w_k = index_convert(INITIAL_WHITE_KING_FILE, 0);
const iSquare b_k = index_convert(INITIAL_BLACK_KING_FILE, NUMBER_OF_RANKS-1);
const iSquare w_k_b = index_convert(1,0);
const iSquare w_k_c = index_convert(2,0);
const iSquare w_k_g = index_convert(6,0);
const iSquare w_k_h = index_convert(7,0);
const iSquare b_k_b = index_convert(1,NUMBER_OF_RANKS-1);
const iSquare b_k_c = index_convert(2,NUMBER_OF_RANKS-1);
const iSquare b_k_g = index_convert(6,NUMBER_OF_RANKS-1);
const iSquare b_k_h = index_convert(7,NUMBER_OF_RANKS-1);

ev_Table_Opening* eval_data_opening = NULL;
ev_Table* eval_data = NULL;

static short glob_piece_values[NUMBER_OF_DIFFERENT_PIECES+2] =
{0, 0, QUEEN_VALUE, ROOK_VALUE, BISHOP_VALUE, KNIGHT_VALUE, PAWN_VALUE, PAWN_VALUE};

const short Black_Center_Value[8][8] = {{0,0,0,0,0,0,HALF_CENTER,0},
								  {0,0,0,0,0,HALF_CENTER,HALF_CENTER,0},
								  {0,0,0,0,HALF_CENTER,HALF_CENTER,HALF_CENTER,0},
								  {0,0,CENTER_PAWN,CENTER_PAWN,CENTER_PAWN,HALF_CENTER,0,0},
								  {0,0,CENTER_PAWN,CENTER_PAWN,CENTER_PAWN,HALF_CENTER,0,0},
								  {0,0,0,HALF_CENTER,HALF_CENTER,HALF_CENTER,HALF_CENTER,0},
								  {0,0,0,0,0,HALF_CENTER,HALF_CENTER,0},
								  {0,0,0,0,0,0,HALF_CENTER,0}};

const short White_Center_Value[8][8] = {{0,HALF_CENTER,0,0,0,0,0,0},
										{0,HALF_CENTER,HALF_CENTER,0,0,0,0,0},
										{0,HALF_CENTER,HALF_CENTER,HALF_CENTER,0,0,0,0},
										{0,0,HALF_CENTER,CENTER_PAWN,CENTER_PAWN,CENTER_PAWN,0,0},
										{0,0,HALF_CENTER,CENTER_PAWN,CENTER_PAWN,CENTER_PAWN,0,0},
										{0,HALF_CENTER,HALF_CENTER,HALF_CENTER,HALF_CENTER,0,0,0},
										{0,HALF_CENTER,HALF_CENTER,0,0,0,0,0},
										{0,HALF_CENTER,0,0,0,0,0,0}};

const short Black_Knight_Value[8][8] = {{8,11,11,11,8,4,3,1},
										{11,16,17,14,10,9,4,3},
										{21,25,25,20,16,12,9,3},
										{25,31,31,24,21,13,9,4},
										{25,31,31,24,21,13,9,4},
										{21,25,25,20,16,12,9,3},
										{11,16,17,14,10,9,4,3},
										{8,11,11,11,8,4,3,1}};

const short White_Knight_Value[8][8] = {{1,3,4,8,11,11,11,8},
										{3,4,9,10,14,17,16,11},
										{3,9,12,16,20,25,25,21},
										{4,9,13,21,24,31,31,25},
										{4,9,13,21,24,31,31,25},
										{3,9,12,16,20,25,25,21},
										{3,4,9,10,14,17,16,11},
										{1,3,4,8,11,11,11,8}};

const short Black_Bishop_Value[8][8] = {{17,17,18,18,17,17,19,17},
										{17,18,19,19,19,20,20,18},
										{17,19,21,19,20,20,18,16},
										{18,19,22,20,20,19,18,16},
										{18,19,22,20,20,19,18,16},
										{17,19,21,19,20,20,18,16},
										{17,18,19,19,19,20,20,18},
										{17,17,18,18,17,17,19,17}};

const short White_Bishop_Value[8][8] = {{17,19,17,17,18,18,17,17},
										{18,20,20,19,19,19,18,17},
										{16,18,20,20,19,21,19,17},
										{16,18,19,20,20,22,19,18},
										{16,18,19,20,20,22,19,18},
										{16,18,20,20,19,21,19,17},
										{18,20,20,19,19,19,18,17},
										{17,19,17,17,18,18,17,17}};

										

/* This is used for determining pawn structures */
const short General_Cover_Mask[8] = {1,3,7,15,31,63,127,255};


short evaluate_opening(situation* pos){
	
	short white_value = 0, black_value = 0;
	char active_color = GET_PIECE_COLOR((*pos).extraInfo);
	
	/* Only if the king is under check: find, whether there are moves */
	find_quickly(pos, eval_table);	

	if((*eval_table).mate)
		if(active_color == WHITE) 
			return -KING_VALUE+MISTAKE_WINDOW*REAL_DEPTH;
		else
			return KING_VALUE-MISTAKE_WINDOW*REAL_DEPTH;

	if((*eval_table).number_of){
		
		char helper = '\0';
		iSquare pos_helper = '\0';
		/*
		char stored_queen_w = NO_CAPTURE, stored_queen_b = NO_CAPTURE;*/
		memset(eval_data_opening, '\0', sizeof(ev_Table_Opening));		

		/* this loop fills the eval_data structure, which will afterwards
		   be used to evaluate the position more accurately */
		for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){

		/* +++	WHITE PIECES +++ */
		pos_helper = (*pos).white_Pieces[i];		
		if(pos_helper != INVALID_SQUARE){
			
			helper = GET_PIECE_TYPE((*pos).square[GET_FILE(pos_helper)]
			[GET_RANK(pos_helper)]);

			white_value += glob_piece_values[helper];
			
			if(helper == PAWN_ID){			
				(*eval_data_opening).pawn_mask_w[GET_FILE(pos_helper)] 
					|=(1 << GET_RANK(pos_helper));
				white_value += White_Center_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
			}

			else if(helper & 4){
				if(helper & 1)
					white_value += White_Knight_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
				else{
					white_value += White_Bishop_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
					(*eval_data_opening).w_bishop_count += 1;
				}
			}
			
			else if(helper == QUEEN_ID){
				++((*eval_data_opening).w_queen_count);
				(*eval_data_opening).w_queen = pos_helper;
			}

			else if(helper == KING_ID)
				(*eval_data_opening).w_king = pos_helper;
		}

		/* BLACK PIECES */
		pos_helper = (*pos).black_Pieces[i];
		
		if(pos_helper != INVALID_SQUARE){
			
			helper = GET_PIECE_TYPE((*pos).square[GET_FILE(pos_helper)]
			[GET_RANK(pos_helper)]);

			black_value += glob_piece_values[helper];
			
			if(helper == PAWN_ID){
				(*eval_data_opening).pawn_mask_b[GET_FILE(pos_helper)] 
					|= (1 << (7-GET_RANK(pos_helper)));
				black_value += Black_Center_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
			}

			else if(helper & 4){
				if(helper & 1)
					black_value += Black_Knight_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
				else{
					black_value += Black_Bishop_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
					(*eval_data_opening).b_bishop_count += 1;
				}
			}
			
			else if(helper == QUEEN_ID){
				++((*eval_data_opening).b_queen_count);
				(*eval_data_opening).b_queen = pos_helper;
			}

			else if(helper == KING_ID)
				(*eval_data_opening).b_king = pos_helper;
		}
		}
		
		if(eval_data_opening->b_bishop_count > eval_data_opening->w_bishop_count)
			black_value += BISHOP_PAIR_OPENING;
		else if(eval_data_opening->w_bishop_count > eval_data_opening->b_bishop_count)
			white_value += BISHOP_PAIR_OPENING;
		/* here we examine whether a side forgot to castle */

		pos_helper = (*eval_data_opening).w_king;
		
		if(pos_helper == w_k_g || pos_helper == w_k_h){
			if(pos->square[7][0] != WHITE_ROOK){
				if(eval_data_opening->pawn_mask_b[7])
					white_value += SAFE_KING;
				if(eval_data_opening->pawn_mask_b[6])
					white_value += SAFE_KING;
				if(pos->square[6][1] == WHITE_PAWN ||
				   pos->square[6][2] == WHITE_PAWN)
					white_value += SAFE_KING;
				if(pos->square[7][1] == WHITE_PAWN ||
					pos->square[7][2] == WHITE_PAWN)
					white_value += SAFE_KING;
			}
		}

		else if(pos_helper == w_k_c || pos_helper == w_k_b){
			if(pos->square[0][0] != WHITE_ROOK && pos->square[1][0] != WHITE_ROOK){
				if(eval_data_opening->pawn_mask_b[0] && 
				   eval_data_opening->pawn_mask_b[2])
					white_value += SAFE_KING;
				if(eval_data_opening->pawn_mask_b[1])
					white_value += SAFE_KING;
				if(pos->square[1][1] == WHITE_PAWN ||
				   pos->square[1][2] == WHITE_PAWN)
					white_value += SAFE_KING;
				if(pos->square[0][1] == WHITE_PAWN &&
				   eval_data_opening->pawn_mask_w[2])
					white_value += SAFE_KING;
			}		
		}

		pos_helper = (*eval_data_opening).b_king;
		
		if(pos_helper == b_k_g || pos_helper == b_k_h){
			if(pos->square[7][7] != BLACK_ROOK){
				if(eval_data_opening->pawn_mask_w[7])
					black_value += SAFE_KING;
				if(eval_data_opening->pawn_mask_w[6])
					black_value += SAFE_KING;
				if(pos->square[6][6] == BLACK_PAWN ||
				   pos->square[6][5] == BLACK_PAWN)
					black_value += SAFE_KING;
				if(pos->square[7][6] == BLACK_PAWN ||
					pos->square[7][5] == BLACK_PAWN)
					black_value += SAFE_KING;
			}
		}

		else if(pos_helper == b_k_c || pos_helper == b_k_b){
			if(pos->square[0][7] != BLACK_ROOK && pos->square[1][7] != BLACK_ROOK){
				if(eval_data_opening->pawn_mask_w[0] && 
				   eval_data_opening->pawn_mask_w[2])
					black_value += SAFE_KING;
				if(eval_data_opening->pawn_mask_w[1])
					black_value += SAFE_KING;
				if(pos->square[1][6] == BLACK_PAWN ||
				   pos->square[1][5] == BLACK_PAWN)
					black_value += SAFE_KING;
				if(pos->square[0][6] == BLACK_PAWN &&
				   eval_data_opening->pawn_mask_b[2])
					black_value += SAFE_KING;
			}		
		}
		
		if((*eval_data_opening).w_queen_count && GET_RANK((*eval_data_opening).w_queen) > 2)
			white_value -= (blackNum + QUEEN_TOO_SOON);
		
		if((*eval_data_opening).b_queen_count && GET_RANK((*eval_data_opening).b_queen) < 5)
			black_value -= (whiteNum + QUEEN_TOO_SOON);
		
		return white_value-black_value + 
			(active_color == WHITE ? SIDE_TO_MOVE_BONUS : -SIDE_TO_MOVE_BONUS);
		}

	return STALE_MATE; 
}




/* this is a very simplistic evaluation function and replaces evaluate_opening
   when the total number of pieces dropped below #define OPENING */
short evaluate(situation* pos_searched){

	short white_value = 0;
	short black_value = 0;
	short white_extra_value = 0;
	short black_extra_value = 0;
	

	find_quickly(pos_searched, eval_table);

	char active_color = GET_PIECE_COLOR((*pos_searched).extraInfo);

	if((*eval_table).mate)
		if(active_color == WHITE) 
			return -KING_VALUE+MISTAKE_WINDOW*REAL_DEPTH;
		else
			return KING_VALUE-MISTAKE_WINDOW*REAL_DEPTH;

	if((*eval_table).number_of){
		
		char helper = '\0', extraHelper = '\0';
		iSquare pos_helper = '\0', pos_helper2 = '\0';
		int total_pieces = whiteNum + blackNum;
		/*
		char stored_queen_w = NO_CAPTURE, stored_queen_b = NO_CAPTURE;*/
		memset(eval_data,'\0',sizeof(ev_Table));		

		/* this loop fills the eval_data structure, which will afterwards
		   be used to evaluate the position more accurately */
		for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){

		pos_helper = (*pos_searched).white_Pieces[i];
		
		if(pos_helper != INVALID_SQUARE){
			
			helper = GET_PIECE_TYPE((*pos_searched).square[GET_FILE(pos_helper)]
			[GET_RANK(pos_helper)]);
			white_value += glob_piece_values[helper];
			
			if(helper == PAWN_ID){
				(*eval_data).pawn_count_w[GET_FILE(pos_helper)] += 1;
				
				/* passed_pawn preparation */
				(*eval_data).pawn_mask_w[GET_FILE(pos_helper)] |=
					(1 << GET_RANK(pos_helper));
				if((*eval_data).pawn_most_advanced_w[GET_FILE(pos_helper)] < GET_RANK(pos_helper))
					(*eval_data).pawn_most_advanced_w[GET_FILE(pos_helper)] = GET_RANK(pos_helper);
				(*eval_data).pawn_count_w_r[GET_RANK(pos_helper)] += 1;
				(*eval_data).pawn_count_w_rank_id[GET_FILE(pos_helper)] =
					GET_RANK(pos_helper);
				(*eval_data).piece_count_w += 1;
				white_extra_value += White_Center_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
			}

			else if(helper & 4){
				(*eval_data).piece_count_w += 1;
				if(helper & 1)
					white_extra_value += White_Knight_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
				else{
					extraHelper = ((*eval_data).w_bishop_count += 1);
					if(extraHelper < 3)
						(*eval_data).w_bishop[extraHelper-1] = pos_helper;
					white_extra_value += White_Bishop_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
				}
			}

			else if(helper == ROOK_ID){
				(*eval_data).rook_count_w[GET_FILE(pos_helper)] += 1;
				(*eval_data).rook_count_w_r[GET_RANK(pos_helper)] += 1;
				(*eval_data).piece_count_w += 1;
				(*eval_data).w_rook = pos_helper;
				(*eval_data).w_rook_count += 1;
			}

			
			else if(helper == QUEEN_ID){
				(*eval_data).piece_count_w += 1;
				(*eval_data).w_queen_count += 1;
				(*eval_data).w_queen = pos_helper;
			}

			else
				(*eval_data).w_king = pos_helper;
		}

		/* Here we deal with black_pieces */
		pos_helper = (*pos_searched).black_Pieces[i];

		if(pos_helper != INVALID_SQUARE){

			helper = GET_PIECE_TYPE((*pos_searched).square[GET_FILE(pos_helper)]
			[GET_RANK(pos_helper)]);

			black_value += glob_piece_values[helper];

			if(helper == PAWN_ID){
				(*eval_data).pawn_count_b[GET_FILE(pos_helper)] += 1;
				
				/* passed_pawn preparation */
				(*eval_data).pawn_mask_b[GET_FILE(pos_helper)] |=
					(1 << (7-GET_RANK(pos_helper)));
				if((*eval_data).pawn_most_advanced_b[GET_FILE(pos_helper)] < 
					(7-GET_RANK(pos_helper)))
					(*eval_data).pawn_most_advanced_b[GET_FILE(pos_helper)] =
						7-GET_RANK(pos_helper);

				(*eval_data).pawn_count_b_r[GET_RANK(pos_helper)] += 1;
				(*eval_data).pawn_count_b_rank_id[GET_FILE(pos_helper)] =
					GET_RANK(pos_helper);
				(*eval_data).piece_count_b += 1;
				black_extra_value += Black_Center_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
			}

			else if(helper & 4){
				(*eval_data).piece_count_b += 1;
				
				if(helper & 1)
					black_extra_value += Black_Knight_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
				else{
					extraHelper = ((*eval_data).b_bishop_count += 1);
					if(extraHelper < 3)
						(*eval_data).b_bishop[extraHelper-1] = pos_helper;
					black_extra_value += Black_Bishop_Value[GET_FILE(pos_helper)][GET_RANK(pos_helper)];
					}
			}

			else if(helper == ROOK_ID){
				(*eval_data).rook_count_b[GET_FILE(pos_helper)] += 1;
				(*eval_data).rook_count_b_r[GET_RANK(pos_helper)] += 1;
				(*eval_data).piece_count_b += 1;
				(*eval_data).b_rook = pos_helper;
				(*eval_data).b_rook_count += 1;
				}

			

			else if(helper == QUEEN_ID){
				(*eval_data).piece_count_b += 1;
				(*eval_data).b_queen_count += 1;
				(*eval_data).b_queen = pos_helper;
			}

			else
				(*eval_data).b_king = pos_helper;
		}
			
		}

		if(eval_data->b_bishop_count > eval_data->w_bishop_count)
			black_value += BISHOP_PAIR_MIDGAME;
		else if(eval_data->w_bishop_count > eval_data->b_bishop_count)
			white_value += BISHOP_PAIR_MIDGAME;
		/* here we examine whether a side forgot to castle */

		/* Passed pawn variables*/ 
		short Forward = 0, passed_bonus = 0;

		/* MID_GAME: OPEN FILES, DANGEROUS_PIECES, PASSED PAWNS, KINGSIDE ATTACK */
		if(total_pieces > 3 || (black_value < 200 && blackNum-1)
			|| (whiteNum-1 && white_value < 200)){
		for(int i = 0; i < NUMBER_OF_RANKS; i++){
			
			/* open files */
			if((*eval_data).rook_count_w[i]){
				if(!(*eval_data).pawn_mask_w[i]){
					white_value += OPEN_FILE;
					if(abs(GET_FILE((*eval_data).b_king)-i) < 2)
						white_value += DANGEROUS_FILE;
				}
			}
			if((*eval_data).rook_count_b[i]){
				if(!(*eval_data).pawn_mask_b[i]){
					black_value += OPEN_FILE;
					if(abs(GET_FILE((*eval_data).w_king)-i) < 2)
						black_value += DANGEROUS_FILE;
				}
			}

			/* open ranks */
			if((*eval_data).rook_count_w_r[i]){
				if(!(*eval_data).pawn_count_w_r[i]){
					white_value += OPEN_RANK;
					if(abs(GET_RANK((*eval_data).b_king)-i) < 2)
						white_value += DANGEROUS_RANK;
				}
			}

			if((*eval_data).rook_count_b_r[i]){
				if(!(*eval_data).pawn_count_b_r[i]){
					black_value += OPEN_RANK;
					if(abs(GET_RANK((*eval_data).w_king)-i) < 2)
							black_value += DANGEROUS_RANK;
				}
			}
			
			
			/* +++ DOUBLED_PAWNS +++ */
			if((*eval_data).pawn_count_w[i] > 1)
					white_value -= MULTIPLE_PAWNS * (32-total_pieces); /* the less pieces there
																	   are, the worse are the
																	   doubled pawns ... */
			if((*eval_data).pawn_count_b[i] > 1)
					black_value -= MULTIPLE_PAWNS * (32-total_pieces);


			/* +++ PASSED PAWNS +++ */
			if(i && i < NUMBER_OF_FILES-1){
				/* First handle the white pawns. */
				if(Forward = (*eval_data).pawn_most_advanced_w[i]){
					if(!((*eval_data).pawn_mask_b[i] & General_Cover_Mask[6-Forward])){
						passed_bonus = 0;
						if(!((*eval_data).pawn_mask_b[i-1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						if(!((*eval_data).pawn_mask_b[i+1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						/* TODO HERE: pay attention to the opponent and own king's position,
						   if this is a pawn ending. */
						if(passed_bonus){
							white_value += ((Forward << 1) + passed_bonus 
											+ General_Cover_Mask[Forward-1]);
							if(blackNum < 9 && !((*eval_data).b_queen_count)){
								white_value += ((abs(GET_FILE((*eval_data).b_king) - i)
												+ 7-GET_RANK((*eval_data).b_king)) << 2);
								white_value -= (abs(GET_FILE((*eval_data).w_king) - i) << 2);
							}
						}
					}
				}
				if(Forward = (*eval_data).pawn_most_advanced_b[i]){
					/* Black pawns here */
					if(!((*eval_data).pawn_mask_w[i] & General_Cover_Mask[6-Forward])){
						passed_bonus = 0;
						if(!((*eval_data).pawn_mask_w[i-1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						if(!((*eval_data).pawn_mask_w[i+1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						if(passed_bonus){
							black_value += ((Forward << 1) + passed_bonus 
										    + General_Cover_Mask[Forward-1]);
							if(whiteNum < 9 && !((*eval_data).w_queen_count)){
								black_value += ((abs(GET_FILE((*eval_data).w_king) - i)
												   + GET_RANK((*eval_data).w_king)) << 2);
								black_value -= (abs(GET_FILE((*eval_data).b_king) - i) << 2);
							}
						}
					}
				}
			}

			
			
			else if(!i){
				/* First handle the white pawns. */
				if(Forward = (*eval_data).pawn_most_advanced_w[i]){
					if(!((*eval_data).pawn_mask_b[i] & General_Cover_Mask[6-Forward])){
						passed_bonus = DANG_PAWN;
						if(!((*eval_data).pawn_mask_b[i+1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						/* TODO HERE: pay attention to the opponent and own king's position,
						   if this is a pawn ending. */
						if(passed_bonus){
							white_value += ((Forward << 1) + passed_bonus 
											+ General_Cover_Mask[Forward-1]);
							if(blackNum && !((*eval_data).b_queen_count)){
								white_value += ((abs(GET_FILE((*eval_data).b_king) - i)
												+ 7-GET_RANK((*eval_data).b_king)) << 2);
								white_value -= (abs(GET_FILE((*eval_data).w_king) - i) << 2);
							}
						}
					}
				}
			

				if(Forward = (*eval_data).pawn_most_advanced_b[i]){
					if(!((*eval_data).pawn_mask_w[i] & General_Cover_Mask[6-Forward])){
						passed_bonus = DANG_PAWN;
						if(!((*eval_data).pawn_mask_w[i+1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						if(passed_bonus){
							black_value += ((Forward << 1) + passed_bonus 
										    + General_Cover_Mask[Forward-1]);
							if(whiteNum < 9 && 
								!((*eval_data).w_queen_count)){
								black_value += ((abs(GET_FILE((*eval_data).w_king) - i)
												   + GET_RANK((*eval_data).w_king)) << 2);
								black_value -= (abs(GET_FILE((*eval_data).b_king) - i) << 2);
							}
						}
					}
				}
			}

			else if(i == NUMBER_OF_FILES-1){
				/* First handle the white pawns. */
				if(Forward = (*eval_data).pawn_most_advanced_w[i]){
					if(!((*eval_data).pawn_mask_b[i] & General_Cover_Mask[6-Forward])){
						passed_bonus = DANG_PAWN;
						if(!((*eval_data).pawn_mask_b[i-1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						/* TODO HERE: pay attention to the opponent and own king's position,
						   if this is a pawn ending. */
						if(passed_bonus){
							white_value += ((Forward << 1) + passed_bonus 
											+ General_Cover_Mask[Forward-1]);
							if(blackNum < 9 && 
								!((*eval_data).b_queen_count)){
								white_value += ((abs(GET_FILE((*eval_data).b_king) - i)
												+ 7-GET_RANK((*eval_data).b_king)) << 2);
								white_value -= (abs(GET_FILE((*eval_data).w_king) - i) << 2);
							}
						}
					}
				}

				if(Forward = (*eval_data).pawn_most_advanced_b[i]){
					if(!((*eval_data).pawn_mask_w[i] & General_Cover_Mask[6-Forward])){
						passed_bonus = DANG_PAWN;
						if(!((*eval_data).pawn_mask_w[i-1] & General_Cover_Mask[6-Forward]))
							passed_bonus += DANG_PAWN;
						if(passed_bonus){
							black_value += ((Forward << 1) + passed_bonus 
										    + General_Cover_Mask[Forward-1]);
							if(whiteNum < 9 && 
								!((*eval_data).w_queen_count)){
								black_value += ((abs(GET_FILE((*eval_data).w_king) - i)
												   + GET_RANK((*eval_data).w_king)) << 2);
								black_value -= (abs(GET_FILE((*eval_data).b_king) - i) << 2);
							}

						}
					}
				}
			}

			if(i < 2){
				
				if(i < (*eval_data).b_bishop_count){
				helper = abs(GET_FILE((*eval_data).b_bishop[i])-GET_FILE((*eval_data).w_king))
					- abs(GET_RANK((*eval_data).b_bishop[i])-GET_RANK((*eval_data).w_king));
				if(abs(helper) < 2)
					black_value += DANG_BISHOP;
				}

				if(i < (*eval_data).w_bishop_count){
				helper = abs(GET_FILE((*eval_data).w_bishop[i])-GET_FILE((*eval_data).b_king))
					- abs(GET_RANK((*eval_data).w_bishop[i])-GET_RANK((*eval_data).b_king));
				if(abs(helper) < 2)
					white_value += DANG_BISHOP;
				}
			}
		}

		

		
		return white_value-black_value + 
			(active_color == WHITE ? SIDE_TO_MOVE_BONUS : -SIDE_TO_MOVE_BONUS);
		}


		
		
		/* mate with queen */
		if(whiteNum == 2){
			if(blackNum == 1){
				if((*eval_data).w_queen_count){
				helper = (*eval_data).b_king;
				pos_helper = (*eval_data).w_king;
				black_value -= QUEEN_MATE_FACTOR*abs((int)(4.5 - (GET_RANK(helper)+1)));
				black_value -= QUEEN_MATE_FACTOR*abs((int)(4.5 - (GET_FILE(helper)+1)));
				black_value += QUEEN_MATE_FACTOR_2*abs(GET_FILE(pos_helper)-GET_FILE(helper));
				black_value += QUEEN_MATE_FACTOR_2*abs(GET_RANK(pos_helper)-GET_RANK(helper));
				return white_value-black_value + 
				(active_color == WHITE ? SIDE_TO_MOVE_BONUS : -SIDE_TO_MOVE_BONUS);
				}
				else if((*eval_data).w_rook_count){
				helper = (*eval_data).b_king;
				pos_helper = (*eval_data).w_king;
				black_value -= ROOK_MATE_FACTOR*abs((int)(4.5 - (GET_RANK(helper)+1)));
				black_value -= ROOK_MATE_FACTOR*abs((int)(4.5 - (GET_FILE(helper)+1)));
				black_value += ROOK_MATE_FACTOR_2*abs(GET_FILE(pos_helper)-GET_FILE(helper));
				black_value += ROOK_MATE_FACTOR_2*abs(GET_RANK(pos_helper)-GET_RANK(helper));
				return white_value-black_value + 
				(active_color == WHITE ? SIDE_TO_MOVE_BONUS : -SIDE_TO_MOVE_BONUS);
				}
			}
		}
				
		if(blackNum == 2){
			if(whiteNum == 1){
				if((*eval_data).b_queen_count){
				helper = (*eval_data).w_king;
				pos_helper = (*eval_data).b_king;
				white_value -= QUEEN_MATE_FACTOR*abs((int)(4.5 - (GET_RANK(helper)+1)));
				white_value -= QUEEN_MATE_FACTOR*abs((int)(4.5 - (GET_FILE(helper)+1)));
				white_value += QUEEN_MATE_FACTOR_2*abs(GET_FILE(pos_helper)-GET_FILE(helper));
				white_value += QUEEN_MATE_FACTOR_2*abs(GET_RANK(pos_helper)-GET_RANK(helper));
				return white_value-black_value + 
				(active_color == WHITE ? SIDE_TO_MOVE_BONUS : -SIDE_TO_MOVE_BONUS);
			 }
				else if((*eval_data).b_rook_count){
				helper = (*eval_data).w_king;
				pos_helper = (*eval_data).b_king;
				white_value -= ROOK_MATE_FACTOR*abs((int)(4.5 - (GET_RANK(helper)+1)));
				white_value -= ROOK_MATE_FACTOR*abs((int)(4.5 - (GET_FILE(helper)+1)));
				white_value += ROOK_MATE_FACTOR_2*abs(GET_FILE(pos_helper)-GET_FILE(helper));
				white_value += ROOK_MATE_FACTOR_2*abs(GET_RANK(pos_helper)-GET_RANK(helper));
				return white_value-black_value + 
				(active_color == WHITE ? SIDE_TO_MOVE_BONUS : -SIDE_TO_MOVE_BONUS);
				}
			}
		}
		/* end of mate with queen (rook) */	
		
		return white_value-black_value + 
			(active_color == WHITE ? SIDE_TO_MOVE_BONUS : -SIDE_TO_MOVE_BONUS);

	
	}	

return STALE_MATE;
}
/* -------------------------------------
		END OF EVALUATION FUNCTIONS
   -------------------------------------*/




/* ---------------------------------------
		GLOBAL MULTI-PURPOSE FUNCTIONS
   ---------------------------------------*/
char GET_FILE(iSquare src){
	return src >> 4;
}

char GET_RANK(iSquare src){
	return src & 7;
}

/* adds a vector to a square */
iSquare add_vector(iSquare start_point, int vec_x, int vec_y){
	if(GET_FILE(start_point) + vec_x < 0 || GET_FILE(start_point) + vec_x > NUMBER_OF_FILES-1 ||
		GET_RANK(start_point) + vec_y < 0 || GET_RANK(start_point) + vec_y > NUMBER_OF_RANKS-1)
		return INVALID_SQUARE;
	
	return index_convert(GET_FILE(start_point)+vec_x,
		GET_RANK(start_point)+vec_y);
}

/* multiplies a vector with a number */
void multiply_vector(chess_vector* source, chess_vector* dest, int scalar){
	(*dest).vec_x = (*source).vec_x * scalar;
	(*dest).vec_y = (*source).vec_y * scalar;
}

int is_corner_square(int file, int rank){
	return (file == 0 && rank == 0) ||
		(file == 0 && rank == NUMBER_OF_RANKS-1) ||
		(file == NUMBER_OF_FILES-1 && rank == 0) ||
		(file == NUMBER_OF_FILES-1 && rank == NUMBER_OF_RANKS-1);
}

chess_vector subtract(iSquare top, iSquare base){
	chess_vector result = {0,0};
	result.vec_x = GET_FILE(top)-GET_FILE(base);
	result.vec_y = GET_RANK(top)-GET_RANK(base);
	return result;
}

/* -------------------------------------------
	  END OF GLOBAL MULTI-PURPOSE FUNCTIONS
   -------------------------------------------*/




/* -----------------------------
	  BASIC INITIALIZATION
   ----------------------------- */
int init_move_tables(){		/* once it becomes interesting to search for
							   better moves by increasing the search
							   depth, there will likely be more than only 1
							   move_table */
	Possible_Moves = (move_table*) calloc(1, sizeof(move_table));
	if(!Possible_Moves)
		return FALSE;
	(*Possible_Moves).number_of = 0;
	

	if(!(eval_table = (move_table*) calloc(1,sizeof(move_table))))
		return FALSE;

	if(!(Try_Board = (situation*) calloc(MAX_DEPTH+2, sizeof(situation))))
		return FALSE; 

	if(!(Try_Table = (move_table*) calloc(MAX_DEPTH+2,sizeof(move_table))))
		return FALSE;

	if(!(depth_pre_sort = (bucket_criteria*) calloc(MAX_DEPTH+2, sizeof(bucket_criteria))))
		return FALSE;

	if(!(hyper_cont = (move_table*) calloc(HYPER_CRITERIA, sizeof(move_table))))
		return FALSE;

	if(!(eval_data = (ev_Table*) calloc(1, sizeof(ev_Table))))
		return FALSE;

	if(!(eval_data_opening = (ev_Table_Opening*) calloc(1,sizeof(ev_Table_Opening))))
		return FALSE;

	if(!init_tree())	/* BINARY TREE */
		return FALSE;

	return TRUE;
}

int create_Destinations(int piece_type, int file, int rank){
	
	int targets = 0;
	int targetCounter = 0;
	iSquare insertion = '\0';
	chess_vector helper = {0,0};
	chess_vector helper2 = {0,0};
	

	switch(piece_type+1){
		case KING_ID: 
			fixed_Destination[piece_type][file][rank][STANDARD_DIR].number_of = 0;

			for(int i=SOUTHWEST; i <= SOUTH; ++i){
				/* CASTLING CONDITIONS !! */
				if(((file == INITIAL_WHITE_KING_FILE &&
					rank == INITIAL_WHITE_KING_RANK) ||
					(file == INITIAL_BLACK_KING_FILE &&
					rank == INITIAL_BLACK_KING_RANK)) &&
					(i == WEST || i == EAST))
						fixed_Destination[piece_type][file][rank][i].number_of = 2;
				else
						fixed_Destination[piece_type][file][rank][i].number_of = 1;
			}

			
			if(!file){
				fixed_Destination[piece_type][file][rank][SOUTHWEST].number_of = 0;
				fixed_Destination[piece_type][file][rank][WEST].number_of = 0;
				fixed_Destination[piece_type][file][rank][NORTHWEST].number_of = 0;
			}

			if(!rank){
				fixed_Destination[piece_type][file][rank][SOUTHWEST].number_of = 0;
				fixed_Destination[piece_type][file][rank][SOUTHEAST].number_of = 0;
				fixed_Destination[piece_type][file][rank][SOUTH].number_of = 0;
			}
			
			if(file == NUMBER_OF_FILES-1){
				fixed_Destination[piece_type][file][rank][NORTHEAST].number_of = 0;
				fixed_Destination[piece_type][file][rank][EAST].number_of = 0;
				fixed_Destination[piece_type][file][rank][SOUTHEAST].number_of = 0;
			}

			if(rank == NUMBER_OF_RANKS-1){
				fixed_Destination[piece_type][file][rank][NORTHWEST].number_of = 0;
				fixed_Destination[piece_type][file][rank][NORTH].number_of = 0;
				fixed_Destination[piece_type][file][rank][NORTHEAST].number_of = 0;
			}


			for(int i=SOUTHWEST; i <= SOUTH; ++i){
				if(fixed_Destination[piece_type][file][rank][i].number_of){
					fixed_Destination[piece_type][file][rank][i].dest_Square =
						(iSquare*) calloc(
						fixed_Destination[piece_type][file][rank][i].number_of,
						sizeof(iSquare));
					if(!fixed_Destination[piece_type][file][rank][i].dest_Square)
						return FALSE;
					
					if(i == SOUTHWEST)
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),-1,-1);
					else if(i == WEST){
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),-1,0);
						if((file == INITIAL_WHITE_KING_FILE &&
							rank == INITIAL_WHITE_KING_RANK) ||
							(file == INITIAL_BLACK_KING_FILE &&
							rank == INITIAL_BLACK_KING_RANK))
							fixed_Destination[piece_type][file][rank][i].dest_Square[1] =
								add_vector(index_convert(file,rank),-2,0);
					}



					else if(i == NORTHWEST)
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),-1,1);
					else if(i == NORTH)
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),0,1);
					else if(i == NORTHEAST)
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),1,1);

					else if(i == EAST){ /* kingside castle */
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),1,0);
						if((file == INITIAL_WHITE_KING_FILE &&
							rank == INITIAL_WHITE_KING_RANK) ||
							(file == INITIAL_BLACK_KING_FILE &&
							rank == INITIAL_BLACK_KING_RANK))
							fixed_Destination[piece_type][file][rank][i].dest_Square[1] =
								add_vector(index_convert(file,rank),2,0);
					}

					else if(i == SOUTHEAST)
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),1,-1);
					else if(i == SOUTH)
						fixed_Destination[piece_type][file][rank][i].dest_Square[0] =
							add_vector(index_convert(file,rank),0,-1);
				}
			}	
			
			break;


		case QUEEN_ID:
			fixed_Destination[piece_type][file][rank][STANDARD_DIR].number_of = 0;
			
			fixed_Destination[piece_type][file][rank][SOUTHWEST].number_of = 
				(file > rank) ? rank : file;
			
			fixed_Destination[piece_type][file][rank][WEST].number_of = 
				file;
			
			fixed_Destination[piece_type][file][rank][NORTHWEST].number_of = 
				(NUMBER_OF_RANKS-rank-1 > file) ? file : (NUMBER_OF_RANKS-rank-1);

			fixed_Destination[piece_type][file][rank][NORTH].number_of = 
				NUMBER_OF_RANKS-rank-1;

			fixed_Destination[piece_type][file][rank][NORTHEAST].number_of =
				(file > rank) ? NUMBER_OF_FILES-file-1 : NUMBER_OF_RANKS-rank-1;

			fixed_Destination[piece_type][file][rank][EAST].number_of = 
				NUMBER_OF_FILES-file-1;

			fixed_Destination[piece_type][file][rank][SOUTHEAST].number_of = 
				(NUMBER_OF_FILES-file-1 > rank) ? rank : (NUMBER_OF_FILES-file-1);

			fixed_Destination[piece_type][file][rank][SOUTH].number_of = 
				rank;

			/* allocates the memory that is needed for every direction of queen moves*/
			for(int i = SOUTHWEST; i <= SOUTH; ++i)
				if(fixed_Destination[piece_type][file][rank][i].number_of){
					fixed_Destination[piece_type][file][rank][i].dest_Square = 
						(iSquare*) calloc(
						fixed_Destination[piece_type][file][rank][i].number_of,
						sizeof(iSquare));
					if(!fixed_Destination[piece_type][file][rank][i].dest_Square)
						return FALSE;

					if(i == SOUTHWEST || i == WEST || i == NORTHWEST)
								helper.vec_x = -1;
					else if(i == NORTH || i == SOUTH)
								helper.vec_x = 0;
					else if(i == NORTHEAST || i == EAST || i == SOUTHEAST)
								helper.vec_x = 1;
					if(i == NORTHWEST || i == NORTH || i == NORTHEAST)
								helper.vec_y = 1;
					else if(i == WEST || i == EAST)
								helper.vec_y = 0;
					else if(i == SOUTHWEST || i == SOUTHEAST || i == SOUTH)
								helper.vec_y = -1;

					
					for(int j=1; j <= fixed_Destination[piece_type][file][rank][i].number_of;
						++j){
							multiply_vector(&helper, &helper2, j);
							insertion = add_vector(index_convert(file,rank),
								helper2.vec_x, helper2.vec_y);
							if(insertion != INVALID_SQUARE)
								fixed_Destination[piece_type][file][rank][i].dest_Square[j-1] =
									insertion;
						}
							

				} /* END OF BIG IF */
				
		
			break;


		case ROOK_ID:

			fixed_Destination[piece_type][file][rank][STANDARD_DIR].number_of = 0;
			
			fixed_Destination[piece_type][file][rank][SOUTHWEST].number_of = 0;
			
			fixed_Destination[piece_type][file][rank][WEST].number_of = 
				file;
			
			fixed_Destination[piece_type][file][rank][NORTHWEST].number_of = 0;

			fixed_Destination[piece_type][file][rank][NORTH].number_of = 
				NUMBER_OF_RANKS-rank-1;

			fixed_Destination[piece_type][file][rank][NORTHEAST].number_of = 0;

			fixed_Destination[piece_type][file][rank][EAST].number_of = 
				NUMBER_OF_FILES-file-1;

			fixed_Destination[piece_type][file][rank][SOUTHEAST].number_of = 0;

			fixed_Destination[piece_type][file][rank][SOUTH].number_of = 
				rank;



			for(int i = WEST; i <= SOUTH; ++i)
				if(fixed_Destination[piece_type][file][rank][i].number_of){
					fixed_Destination[piece_type][file][rank][i].dest_Square = 
						(iSquare*) calloc(
						fixed_Destination[piece_type][file][rank][i].number_of,
						sizeof(iSquare));
					if(!fixed_Destination[piece_type][file][rank][i].dest_Square)
						return FALSE;

					if(i == WEST)
						helper.vec_x = -1;
					else if(i == NORTH || i == SOUTH)
								helper.vec_x = 0;
					else if(i == EAST)
								helper.vec_x = 1;
					
					if(i == NORTH)
							helper.vec_y = 1;
					else if(i == WEST || i == EAST)
								helper.vec_y = 0;
					else if(i == SOUTH)
								helper.vec_y = -1;

					
					for(int j=1; j <= fixed_Destination[piece_type][file][rank][i].number_of;
						++j){
							multiply_vector(&helper, &helper2, j);
							insertion = add_vector(index_convert(file,rank),
								helper2.vec_x, helper2.vec_y);
							if(insertion != INVALID_SQUARE)
								fixed_Destination[piece_type][file][rank][i].dest_Square[j-1] =
									insertion;
						}
							

				} /* END OF BIG IF */

			break;

		case BISHOP_ID:

			fixed_Destination[piece_type][file][rank][STANDARD_DIR].number_of = 0;
			
			fixed_Destination[piece_type][file][rank][SOUTHWEST].number_of = 
				(file > rank) ? rank : file;
			
			fixed_Destination[piece_type][file][rank][WEST].number_of = 
				0;
			
			fixed_Destination[piece_type][file][rank][NORTHWEST].number_of = 
				(NUMBER_OF_RANKS-rank-1 > file) ? file : (NUMBER_OF_RANKS-rank-1);

			fixed_Destination[piece_type][file][rank][NORTH].number_of = 
				0;

			fixed_Destination[piece_type][file][rank][NORTHEAST].number_of =
				(file > rank) ? NUMBER_OF_FILES-file-1 : NUMBER_OF_RANKS-rank-1;

			fixed_Destination[piece_type][file][rank][EAST].number_of = 
				0;

			fixed_Destination[piece_type][file][rank][SOUTHEAST].number_of = 
				(NUMBER_OF_FILES-file-1 > rank) ? rank : (NUMBER_OF_FILES-file-1);

			fixed_Destination[piece_type][file][rank][SOUTH].number_of = 
				0;


			for(int i = SOUTHWEST; i <= SOUTHEAST; i++)
				if(fixed_Destination[piece_type][file][rank][i].number_of){
					fixed_Destination[piece_type][file][rank][i].dest_Square = 
						(iSquare*) calloc(
						fixed_Destination[piece_type][file][rank][i].number_of,
						sizeof(iSquare));
					if(!fixed_Destination[piece_type][file][rank][i].dest_Square)
						return FALSE;

					if(i == SOUTHWEST || i == NORTHWEST)
								helper.vec_x = -1;
					else if(i == NORTHEAST || i == SOUTHEAST)
								helper.vec_x = 1;
					
					if(i == NORTHWEST || i == NORTHEAST)
								helper.vec_y = 1;
					
					else if(i == SOUTHWEST || i == SOUTHEAST)
								helper.vec_y = -1;

					
					for(int j=1; j <= fixed_Destination[piece_type][file][rank][i].number_of;
						++j){
							multiply_vector(&helper, &helper2, j);
							insertion = add_vector(index_convert(file,rank),
								helper2.vec_x, helper2.vec_y);
							if(insertion != INVALID_SQUARE)
								fixed_Destination[piece_type][file][rank][i].dest_Square[j-1] =
									insertion;
						}
							

				} /* END OF BIG IF */
				
		
			break;


		case KNIGHT_ID:
			if(is_corner_square(file,rank))
				targets = 2;

			else if((file == 0 && (rank == 1 || rank == NUMBER_OF_RANKS-2)) ||
				(file == NUMBER_OF_FILES-1 && (rank == 1 || rank == NUMBER_OF_RANKS-2)) ||
				(rank == 0 && (file == 1 || file == NUMBER_OF_FILES-2)) ||
				(rank == NUMBER_OF_RANKS-1 && (file == 1 || file == NUMBER_OF_FILES-2)))
				targets = 3;

			else if(file == 0 || file == NUMBER_OF_FILES-1 ||
				rank == 0 || rank == NUMBER_OF_RANKS-1 ||
				(file == 1 && (rank == 1 || rank == NUMBER_OF_RANKS-2)) ||
				(file == NUMBER_OF_FILES-2 && (rank == 1 || rank == NUMBER_OF_RANKS-2)))
				targets = 4;

			else if((file == 1 && rank > 1 && rank < NUMBER_OF_RANKS-2) ||
				(file == NUMBER_OF_FILES-2 && rank > 1 && rank < NUMBER_OF_RANKS-2) ||
				(rank == 1 && file > 1 && file < NUMBER_OF_FILES-2) ||
				(rank == NUMBER_OF_RANKS-2 && file > 1 && file < NUMBER_OF_FILES-2))
				targets = 6;

			else
				targets = 8;


			fixed_Destination[piece_type][file][rank][STANDARD_DIR].number_of = targets;
			
			fixed_Destination[piece_type][file][rank][STANDARD_DIR].dest_Square = 
				(iSquare*) calloc(targets,sizeof(iSquare));
			
			if(fixed_Destination[piece_type][file][rank][STANDARD_DIR].dest_Square == NULL)
				return FALSE;


			for(int i = -1; i < 2; i+=2)
				for(int j = -2; j < 3; j+=4){
					insertion = add_vector(index_convert(file,rank), i, j);
					if(insertion != INVALID_SQUARE){
						fixed_Destination[piece_type][file][rank][STANDARD_DIR].dest_Square[targetCounter] = insertion;
						targetCounter++;
					}
				}

			for(int i = -2; i < 3; i+=4)
				for(int j = -1; j < 2; j+=2){
					insertion = add_vector(index_convert(file,rank), i, j);
					if(insertion != INVALID_SQUARE){
						fixed_Destination[piece_type][file][rank][STANDARD_DIR].dest_Square[targetCounter] = insertion;
						targetCounter++;
					}
				}

			/* DONE: Filled Array of possible destination squares of a knight on
			   a given square */

			break;


		case PAWN_ID:
			fixed_Destination[piece_type][file][rank][STANDARD_DIR].number_of = 
				0;
			fixed_Destination[piece_type][file][rank][WEST].number_of = 
				0;
			fixed_Destination[piece_type][file][rank][EAST].number_of = 
				0;
			
			if(rank == 1)
				fixed_Destination[piece_type][file][rank][NORTH].number_of = 
				2;
			if(rank != 1 && rank != NUMBER_OF_RANKS-1)
				fixed_Destination[piece_type][file][rank][NORTH].number_of = 
				1;
			if(rank == NUMBER_OF_RANKS-1)
				fixed_Destination[piece_type][file][rank][NORTH].number_of = 
				0;

			if(rank == NUMBER_OF_RANKS-2)
				fixed_Destination[piece_type][file][rank][SOUTH].number_of = 
				2;
			if(rank != NUMBER_OF_RANKS-2 && rank != 0)
				fixed_Destination[piece_type][file][rank][SOUTH].number_of = 
				1;
			if(rank == 0)
				fixed_Destination[piece_type][file][rank][SOUTH].number_of = 
				0;

			if(file == 0){
				fixed_Destination[piece_type][file][rank][NORTHWEST].number_of = 
				0;
				fixed_Destination[piece_type][file][rank][SOUTHWEST].number_of = 
				0;
			}
			
			if(file != 0 && rank != NUMBER_OF_RANKS-1 && rank !=0){
				fixed_Destination[piece_type][file][rank][NORTHWEST].number_of = 
				1;
				fixed_Destination[piece_type][file][rank][SOUTHWEST].number_of = 
				1;
			}

			if(file==NUMBER_OF_FILES-1){
				fixed_Destination[piece_type][file][rank][NORTHEAST].number_of = 
				0;
				fixed_Destination[piece_type][file][rank][SOUTHEAST].number_of = 
				0;
			}

			if(file != NUMBER_OF_FILES-1 && rank != NUMBER_OF_RANKS-1 && rank !=0){
				fixed_Destination[piece_type][file][rank][NORTHEAST].number_of = 
				1;
				fixed_Destination[piece_type][file][rank][SOUTHEAST].number_of = 
				1;
			}

			for(int i=SOUTHWEST; i <= SOUTH; ++i)
				if(fixed_Destination[piece_type][file][rank][i].number_of){
					fixed_Destination[piece_type][file][rank][i].dest_Square =
						(iSquare*) calloc(
						fixed_Destination[piece_type][file][rank][i].number_of,
						sizeof(iSquare));
					if(!fixed_Destination[piece_type][file][rank][i].dest_Square)
						return FALSE;
					
					if(i == SOUTHWEST)
						fixed_Destination[piece_type][file][rank][SOUTHWEST].dest_Square[0]=
								add_vector(index_convert(file,rank),-1,-1);
					
					else if(i == NORTHWEST)
						fixed_Destination[piece_type][file][rank][NORTHWEST].dest_Square[0]=
								add_vector(index_convert(file,rank),-1,1);
					
					else if(i == NORTHEAST)
						fixed_Destination[piece_type][file][rank][NORTHEAST].dest_Square[0]=
								add_vector(index_convert(file,rank),1,1);

					else if(i == SOUTHEAST)
						fixed_Destination[piece_type][file][rank][SOUTHEAST].dest_Square[0]=
								add_vector(index_convert(file,rank),1,-1);

					else if(i == NORTH &&
						fixed_Destination[piece_type][file][rank][NORTH].number_of == 1)
						fixed_Destination[piece_type][file][rank][NORTH].dest_Square[0]=
								add_vector(index_convert(file,rank),0,1);

					else if(i == NORTH &&
						fixed_Destination[piece_type][file][rank][NORTH].number_of == 2){
						fixed_Destination[piece_type][file][rank][NORTH].dest_Square[0]=
								add_vector(index_convert(file,rank),0,1);
						fixed_Destination[piece_type][file][rank][NORTH].dest_Square[1]=
								add_vector(index_convert(file,rank),0,2);
						}

					else if(i == SOUTH &&
						fixed_Destination[piece_type][file][rank][SOUTH].number_of == 1)
						fixed_Destination[piece_type][file][rank][SOUTH].dest_Square[0]=
								add_vector(index_convert(file,rank),0,-1);

					else if(i == SOUTH &&
						fixed_Destination[piece_type][file][rank][SOUTH].number_of == 2){
						fixed_Destination[piece_type][file][rank][SOUTH].dest_Square[0]=
								add_vector(index_convert(file,rank),0,-1);
						fixed_Destination[piece_type][file][rank][SOUTH].dest_Square[1]=
								add_vector(index_convert(file,rank),0,-2);
						}
				}

				break;

				} /* End of switch */

return TRUE;
} /* End of function create_Destinations() */
				

				
int init(){
	int firstCall = TRUE;
	if(CurrentBoard){
		free(CurrentBoard);
		firstCall = FALSE;
	}

	CurrentBoard = (situation*) malloc(sizeof(situation));
	if(!CurrentBoard)
		return FALSE;

	if(!firstCall)
		return TRUE;

	Reference = (situation*) calloc(sizeof(situation),NUMBER_OF_REFERENCES);
	if(!Reference)
		return FALSE; 

	if(!init_move_tables())
		return FALSE;

	fixed_Destination = (Destination****) calloc(NUMBER_OF_DIFFERENT_PIECES,
		sizeof(Destination***));
	if(!fixed_Destination)
		return FALSE;

	int ok_indicator = TRUE;

	for(int i=0; i < NUMBER_OF_DIFFERENT_PIECES; ++i){
		fixed_Destination[i] = (Destination***) calloc(NUMBER_OF_FILES,sizeof(Destination**));
		if(!fixed_Destination[i])
			return FALSE;
		else{
			for(int j=0; j < NUMBER_OF_FILES; ++j){
				fixed_Destination[i][j] = (Destination**) 
					calloc(NUMBER_OF_RANKS,sizeof(Destination*));
				if(!fixed_Destination[i][j])
					return FALSE;
				else{
					for(int k=0; k < NUMBER_OF_RANKS; ++k){
						fixed_Destination[i][j][k] = (Destination*)
							calloc(NUMBER_OF_DIRECTIONS,sizeof(Destination));
						if(!fixed_Destination[i][j][k])
							return FALSE; /* could be additional calloc is necessary here */
						
						ok_indicator = create_Destinations(i,j,k);
						if(!ok_indicator)
							return FALSE;
						
					}
				}
			}
		}
	}

	return TRUE;
}


void move_to_alg(move* src, char* destString){
	destString[0] = GET_FILE((*src).start) + 'a';
	destString[1] = GET_RANK((*src).start) + '1';
	destString[2] = GET_FILE((*src).dest) + 'a';
	destString[3] = GET_RANK((*src).dest) + '1';
	if((*src).promote){
		if(GET_PIECE_TYPE((*src).promote) == QUEEN_ID)
			destString[4] = 'q';
		else if(GET_PIECE_TYPE((*src).promote) == ROOK_ID)
			destString[4] = 'r';
		else if(GET_PIECE_TYPE((*src).promote) == BISHOP_ID)
			destString[4] = 'b';
		else if(GET_PIECE_TYPE((*src).promote) == KNIGHT_ID)
			destString[4] = 'n';
		destString[5] = '\0';
	}
	else
		destString[4] = '\0';
}


/* start of the fenstring converter */
int create_internal_board(char* fenString){
	int result = TRUE;
	
	if(!(fenString && CurrentBoard))
		return FALSE;

	memset(CurrentBoard,'\0',sizeof(situation));

	unsigned fLength = strlen(fenString);
	int endOfSquares = 0, currFile = 0, currRank = 7;
	int nop_white = 0, nop_black = 0;
	char sqDesc = '\0';

	memset((*CurrentBoard).white_Pieces, INVALID_SQUARE, MAXIMUM_NUMBER_OF_PIECES*sizeof(iSquare));
	memset((*CurrentBoard).black_Pieces, INVALID_SQUARE, MAXIMUM_NUMBER_OF_PIECES*sizeof(iSquare));


	/* examines that part of the fenString, which contains info of the squares */
	for(unsigned i=0; i < fLength; i++, sqDesc = '\0'){
		
		if(fenString[i] >= '1' && fenString[i] <= '8'){
			currFile = (currFile + fenString[i]-'1'+1) % NUMBER_OF_FILES;
			continue;
		}
		
		if(fenString[i] == '/' && currRank > 0){
			--currRank;
			continue;
		}

		if(fenString[i] == ' '){
			endOfSquares = i;
			break;
		}

		
		if(fenString[i] == 'k' || fenString[i] == 'K')
			sqDesc |= KING_ID;
		else if(fenString[i] == 'q' || fenString[i] == 'Q')
			sqDesc |= QUEEN_ID;
		else if(fenString[i] == 'r' || fenString[i] == 'R')
			sqDesc |= ROOK_ID;
		else if(fenString[i] == 'b' || fenString[i] == 'B')
			sqDesc |= BISHOP_ID;
		else if(fenString[i] == 'n' || fenString[i] == 'N')
			sqDesc |= KNIGHT_ID;
		else if(fenString[i] == 'p' || fenString[i] == 'P')
			sqDesc |= PAWN_ID;
		
		
		if(!sqDesc){
			result = FALSE;
			break;
		}		
		
		
		if(fenString[i] >= 'A' && fenString[i] <= 'Z')
			sqDesc |= WHITE;
		else if(fenString[i] >= 'a' && fenString[i] <= 'z')
			sqDesc |= BLACK;

		

		if(currFile < NUMBER_OF_FILES && currRank >= 0){
			(*CurrentBoard).square[currFile][currRank] = sqDesc;
			if(nop_white < MAXIMUM_NUMBER_OF_PIECES &&
				fenString[i] >= 'A' && fenString[i] <= 'Z'){
					(*CurrentBoard).white_Pieces[nop_white] = 
						index_convert(currFile, currRank);
					if(fenString[i] == 'K')
						(*CurrentBoard).king_w = nop_white;
					nop_white++;
				}
			else if(nop_white >= MAXIMUM_NUMBER_OF_PIECES &&
				fenString[i] >= 'A' && fenString[i] <= 'Z'){
					result = FALSE;
					break;
				}

			if(nop_black < MAXIMUM_NUMBER_OF_PIECES &&
				fenString[i] >= 'a' && fenString[i] <= 'z'){
					(*CurrentBoard).black_Pieces[nop_black] =
						index_convert(currFile, currRank);
					if(fenString[i] == 'k')
						(*CurrentBoard).king_b = nop_black;
					nop_black++;
				}

			else if(nop_black >= MAXIMUM_NUMBER_OF_PIECES &&
				fenString[i] >= 'a' && fenString[i] <= 'z'){
					result = FALSE;
					break;
				}

			currFile = (currFile+1) % NUMBER_OF_FILES;
		}

		
	} /* End of a for_Loop */

	/* Side to move */
	if(result && endOfSquares < fLength-1){
		++endOfSquares;
		if(fenString[endOfSquares] == 'b' || fenString[endOfSquares] == 'B')
			(*CurrentBoard).extraInfo |= BLACK;
		else if(fenString[endOfSquares] == 'w' || fenString[endOfSquares] == 'W')
			(*CurrentBoard).extraInfo |= WHITE;
		else
			result = FALSE;
		++endOfSquares; /* now we are at the ' ' character after b _ w */
	}
	

	/* Castling availabilities */
	if(result && endOfSquares < fLength){
		++endOfSquares;
		while(fenString[endOfSquares] != ' '){
			if(fenString[endOfSquares] == 'K')
				(*CurrentBoard).extraInfo |= WKCASTLE;
			else if(fenString[endOfSquares] == 'Q')
				(*CurrentBoard).extraInfo |= WQCASTLE;
			else if(fenString[endOfSquares] == 'k')
				(*CurrentBoard).extraInfo |= BKCASTLE;
			else if(fenString[endOfSquares] == 'q')
				(*CurrentBoard).extraInfo |= BQCASTLE;
			++endOfSquares;
			if(endOfSquares >= fLength){
				result = FALSE;
				break;
			}
		}
	}
	




	/* en passant Square */
	if(result && endOfSquares < fLength-1){
		++endOfSquares;
		if(fenString[endOfSquares] != '-' && endOfSquares < fLength-1){
			if(fenString[endOfSquares+1] == '3' || 
				fenString[endOfSquares+1] == '6'){
				(*CurrentBoard).epSquare = alg_convert(fenString[endOfSquares],
								fenString[endOfSquares+1]);
				endOfSquares += 2;
				}

			else
				result = FALSE;
		}
		else if(fenString[endOfSquares] == '-'){
			(*CurrentBoard).epSquare = INVALID_SQUARE;/* there is no en passant square*/
			++endOfSquares;
		}
	}
	
	

	/* half moves */
	char dec1 = 0, dec2 = 0;
	if(result && endOfSquares < fLength-2){
		++endOfSquares;
		if(fenString[endOfSquares] >= '1' && fenString[endOfSquares] <= '9'){
			dec2 = fenString[endOfSquares] - '1' + 1;
			if(fenString[endOfSquares+1] == '0'){
				dec1 = 10*dec2;
				dec2 = 0;
			}
			else if(fenString[endOfSquares+1] >= '1' &&
				fenString[endOfSquares+1] <= '9'){
					dec1 = 10*dec2;
					dec2 = fenString[endOfSquares+1] - '1' + 1;
				}
			(*CurrentBoard).hMoves = dec1 + dec2;
		}
	}
		
	return result;

}



iSquare alg_convert(char aFile, char aRank){
	return ((0 | (aFile - 'a')) << 4) | (aRank - '1');
}

iSquare index_convert(int iFile, int iRank){
	return ((0 | iFile) << 4) | iRank;
}


void convert_to_alg(iSquare src, char* destFile, char* destRank){
	*destFile = (char) (GET_FILE(src) + 'a');
	*destRank = (char) (GET_RANK(src) + '1');
}

void convert_to_string(iSquare src, char* buffer){
	buffer[0] = (char) (GET_FILE(src) + 'a');
	buffer[1] = (char) (GET_RANK(src) + '1');
	buffer[2] = '\0';
}



void alg_to_move(char* src, move* destMove){
	(*destMove).start = index_convert(src[0]-'a',src[1]-'1');
	(*destMove).dest = index_convert(src[2]-'a',src[3]-'1');
	if(src[4] == 'q' || src[4] == 'Q')
		(*destMove).promote = QUEEN_ID;
	else if(src[4] == 'n' || src[4] == 'N')
		(*destMove).promote = KNIGHT_ID;
	else if(src[4] == 'r' || src[4] == 'R')
		(*destMove).promote = ROOK_ID;
	else if(src[4] == 'b'|| src[4] == 'B')
		(*destMove).promote = BISHOP_ID;
	else
		(*destMove).promote = '\0';

}


/* this is used for moves which are only partially converted by alg_to_move */
int get_Move_index(move* question, move_table* setOfMoves){
	int result = -1;

	for(int i=0; i < (*setOfMoves).number_of; ++i)
		if((*question).start == (*setOfMoves).stored_move[i].start
			&& (*question).dest == (*setOfMoves).stored_move[i].dest
			&& (*question).promote == (*setOfMoves).stored_move[i].promote){
				result = i;
				break;
			}
	return result;
}
/* -----------------------------------------
   ---- END OF INITIALIZATION FUNCTIONS ----
   ----------------------------------------- */





/* ---------------------------------
		SORTING FUNCTIONS + DATA
   ---------------------------------*/
/* This array stores the indizes of the criteria container
   for capturing moves based on [attacker][captured] */
const unsigned 
	capture_idx[NUMBER_OF_DIFFERENT_PIECES+1][NUMBER_OF_DIFFERENT_PIECES+1] = 
			{{0,0,0,0,0,0,0},
			{0,0,1,6,10,10,14},
			{0,0,5,9,13,13,17},
			{0,0,3,8,12,12,16},
			{0,0,4,7,11,11,15},
			{0,0,4,7,11,11,15},
			{0,0,1,6,10,10,14}};

static situation hyper_check;
#define ATTACK_ON_QUEEN 19
#define QUEEN_PROTECT 18
#define ATTACK_ON_ROOK 21
#define ROOK_PROTECT 20
#define ATTACK_ON_LIGHT 23
#define LIGHT_PROTECT 22
#define OTHER_MOVE 24


/* New sorting function since July 11, 2009 */
void hyper_sort(short tab_idx){
	
	iSquare* opp_piece = NULL, *my_piece = NULL;
	int prom_idx[] = {2, 2, 2, 10, 10, 10, 2, 2}; /* err ? */
	int opp_king = 0, cap_help = NO_CAPTURE, 
		active_piece = 0, captured_piece = 0,
		resCont = 0, 
		opp_rook_count = 0,
		opp_light_count = 0,
		queen_in_danger = FALSE,
		rook_in_danger = FALSE,
		own_light_count = 0,
		light_in_danger = 0;

	iSquare oKingSq = '\0', 
			active_help = INVALID_SQUARE,
			dest_help = INVALID_SQUARE,
			iter_help = INVALID_SQUARE,
			opp_queen = INVALID_SQUARE,
			own_queen = INVALID_SQUARE,
			own_rook = INVALID_SQUARE; /* Attacks on the opponent's queen */
	iSquare own_light[4] = {INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE};
	
	iSquare opp_rook[2] = {INVALID_SQUARE, INVALID_SQUARE};
	iSquare opp_light[4] = {INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE};
	char act_color = GET_PIECE_COLOR(Try_Board[tab_idx].extraInfo);

	if(act_color == WHITE){
		my_piece = Try_Board[tab_idx].white_Pieces;
		opp_piece = Try_Board[tab_idx].black_Pieces;
		opp_king = Try_Board[tab_idx].king_b;
		oKingSq = Try_Board[tab_idx].black_Pieces[opp_king];
	}
	else{
		my_piece = Try_Board[tab_idx].black_Pieces;
		opp_piece = Try_Board[tab_idx].white_Pieces;
		opp_king = Try_Board[tab_idx].king_w;
		oKingSq = Try_Board[tab_idx].white_Pieces[opp_king];
	}

	for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){
		iter_help = opp_piece[i];
		if(iter_help != INVALID_SQUARE){
			captured_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(iter_help)]
				[GET_RANK(iter_help)]);			
			if((captured_piece == BISHOP_ID || captured_piece == KNIGHT_ID) &&
				opp_light_count < 4){
				opp_light[opp_light_count] = iter_help;
				++opp_light_count;
			}
			else if(captured_piece == ROOK_ID && opp_rook_count < 2){
				opp_rook[opp_rook_count] = iter_help;
				++opp_rook_count;
			}
			else if(captured_piece == QUEEN_ID)
				opp_queen = iter_help;
			
		}
		iter_help = my_piece[i];
		if(iter_help != INVALID_SQUARE){
			captured_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(iter_help)]
				[GET_RANK(iter_help)]);
				if((captured_piece == KNIGHT_ID || captured_piece == BISHOP_ID)
					&& own_light_count < 4){
						if(quick_check(Try_Board+tab_idx, iter_help)){
							own_light[own_light_count] = iter_help;
							++own_light_count;
							light_in_danger = own_light_count;
						}
				}
				else if(captured_piece == ROOK_ID){
					if(rook_in_danger = quick_check(Try_Board+tab_idx, iter_help))
						own_rook = iter_help;
				}
				else if(captured_piece == QUEEN_ID){
					own_queen = iter_help;
					queen_in_danger = quick_check(Try_Board+tab_idx,own_queen);
				}

		}
	}

	for(int i = 0; i < HYPER_CRITERIA; ++i)
		hyper_cont[i].number_of = 0;	
	
	for(int i = 0; i < Try_Table[tab_idx].number_of; ++i){
		cap_help = Try_Table[tab_idx].stored_move[i].captured_id;
		active_help = Try_Table[tab_idx].stored_move[i].start;
		active_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(active_help)]
			[GET_RANK(active_help)]);
		resCont = OTHER_MOVE;

		/* First: look_for_check */
		memcpy(&hyper_check, Try_Board+tab_idx, sizeof(situation));
		execute_move(&hyper_check, Try_Table[tab_idx].stored_move+i);
		if(quick_check(&hyper_check, oKingSq))
			resCont = 0;
		else if(Try_Table[tab_idx].stored_move[i].promote)
			resCont = prom_idx[(Try_Table[tab_idx].stored_move[i].promote & 7)];


		else if(cap_help != NO_CAPTURE){
			if(captured_piece = 
				GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(opp_piece[cap_help])]
							[GET_RANK(opp_piece[cap_help])]))
					resCont = capture_idx[active_piece][captured_piece];
		}
		else{
			dest_help = Try_Table[tab_idx].stored_move[i].dest;
			/* TODO: ANDERE ZUEGE ALS SCHLAGZUEGE */
			if(quick_check(&hyper_check, opp_queen))
					resCont = ATTACK_ON_QUEEN;
			
			else if(queen_in_danger 
				&& active_piece == QUEEN_ID 
				&& !quick_check(Try_Board+tab_idx, dest_help))
					resCont = QUEEN_PROTECT;

			else if(quick_check(&hyper_check, opp_rook[0]) 
				|| quick_check(&hyper_check, opp_rook[1]))
					resCont = ATTACK_ON_ROOK;

			else if(rook_in_danger
				&& active_help == own_rook
				&& !quick_check(Try_Board+tab_idx, dest_help))
					resCont = ROOK_PROTECT;

			else if(opp_light_count){
				for(int k = 0; k < opp_light_count; ++k)
					if(quick_check(&hyper_check, opp_light[k])){
						resCont = ATTACK_ON_LIGHT;
						break;
					}
			}
			else if(light_in_danger){
				if(active_help == own_light[light_in_danger-1]
				  && !quick_check(Try_Board+tab_idx, dest_help))
					  resCont = LIGHT_PROTECT;
			}
		}		

		hyper_cont[resCont].stored_move[hyper_cont[resCont].number_of] =
			Try_Table[tab_idx].stored_move[i];
		hyper_cont[resCont].number_of = hyper_cont[resCont].number_of + 1;

	} /* END OF for-loop */		
				
	int helper = 0;
	for(int i = 0; i < Try_Table[tab_idx].number_of;){
		for(int j = 0; j < hyper_cont[helper].number_of; ++j){
			Try_Table[tab_idx].stored_move[i] =
				hyper_cont[helper].stored_move[j];
			++i;
		}
		++helper;
	}
}



/* This function is considered to sort more accurately than bucket sort,
   but less accurately than hyper_sort. It could be useful at search depths
   5-6 */
void mediocre_sort(short tab_idx){
		
	iSquare* opp_piece = NULL;
	int prom_idx[] = {2, 2, 2, 10, 10, 10, 2, 2}; /* err ? */
	int opp_king = 0, cap_help = NO_CAPTURE, active_piece = 0, captured_piece = 0,
		resCont = 0;

	iSquare oKingSq = '\0', 
			active_help = INVALID_SQUARE,
			dest_help = INVALID_SQUARE;
			
	char act_color = GET_PIECE_COLOR(Try_Board[tab_idx].extraInfo);

	if(act_color == WHITE){
		opp_piece = Try_Board[tab_idx].black_Pieces;
		opp_king = Try_Board[tab_idx].king_b;
		oKingSq = Try_Board[tab_idx].black_Pieces[opp_king];
	}
	else{
		opp_piece = Try_Board[tab_idx].white_Pieces;
		opp_king = Try_Board[tab_idx].king_w;
		oKingSq = Try_Board[tab_idx].white_Pieces[opp_king];
	}

	for(int i = 0; i < MEDIOCRE_CRITERIA; ++i)
		hyper_cont[i].number_of = 0;	
	
	for(int i = 0; i < Try_Table[tab_idx].number_of; ++i){
		cap_help = Try_Table[tab_idx].stored_move[i].captured_id;
		active_help = Try_Table[tab_idx].stored_move[i].start;
		active_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(active_help)]
			[GET_RANK(active_help)]);
		resCont = MEDIOCRE_MOVE;
		
		if(Try_Table[tab_idx].stored_move[i].promote)
			resCont = prom_idx[(Try_Table[tab_idx].stored_move[i].promote & 7)];


		else if(cap_help != NO_CAPTURE){
			if(captured_piece = 
				GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(opp_piece[cap_help])]
							[GET_RANK(opp_piece[cap_help])]))
					resCont = capture_idx[active_piece][captured_piece];
		}
		

		hyper_cont[resCont].stored_move[hyper_cont[resCont].number_of] =
			Try_Table[tab_idx].stored_move[i];
		hyper_cont[resCont].number_of = hyper_cont[resCont].number_of + 1;

	} /* END OF for-loop */		
				
	int helper = 0;
	for(int i = 0; i < Try_Table[tab_idx].number_of;){
		for(int j = 0; j < hyper_cont[helper].number_of; ++j){
			Try_Table[tab_idx].stored_move[i] =
				hyper_cont[helper].stored_move[j];
			++i;
		}
		++helper;
	}
}


int white_sort(const void* move_1, const void* move_2){
	return ((move*) move_2)->quality - ((move*) move_1)->quality;
}

int black_sort(const void* move_1, const void* move_2){
	return ((move*) move_1)->quality - ((move*) move_2)->quality;
}


#define QUEEN_CAPTURE 0
#define SOME_PROMOTE 1
#define ROOK_CAPTURE 2
#define LIGHT_CAPTURE 3
#define PAWN_CAPTURE 4
#define BUCK_EXTRA 5
#define BUCK_ELSE 6

/* New Version of buck_sort. Slightly more slowly than the old one */
void buck_sort(short tab_idx){

	char act_color = GET_PIECE_COLOR(Try_Board[tab_idx].extraInfo);
	iSquare* opp_piece = NULL;
	short opp_king = 0, resultCont = 0, captured_piece = 0;

	if(act_color == WHITE){
		opp_piece = Try_Board[tab_idx].black_Pieces;
		opp_king = Try_Board[tab_idx].king_b;
	}
	else{
		opp_piece = Try_Board[tab_idx].white_Pieces;
		opp_king = Try_Board[tab_idx].king_w;
	}

	for(int i = 0; i < MEDIOCRE_CRITERIA; ++i)
		depth_pre_sort[tab_idx][i].number_of = 0; /* OLD: pre_sort[i].number_of = 0;*/

	short cap_help = NO_CAPTURE;
	iSquare active_help = INVALID_SQUARE;
	short active_piece = 0;
	
	for(int i = 0; i < Try_Table[tab_idx].number_of; ++i){
		cap_help = Try_Table[tab_idx].stored_move[i].captured_id;
		active_help = Try_Table[tab_idx].stored_move[i].start;
		active_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(active_help)]
			[GET_RANK(active_help)]);
		resultCont = MEDIOCRE_MOVE;
		
		if(Try_Table[tab_idx].stored_move[i].promote)
			resultCont = 2;
		
		else if(cap_help != NO_CAPTURE){
			if(captured_piece = 
				GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(opp_piece[cap_help])]
							[GET_RANK(opp_piece[cap_help])]))
					resultCont = capture_idx[active_piece][captured_piece];
		}

		depth_pre_sort[tab_idx][resultCont].stored_move[depth_pre_sort[tab_idx][resultCont].number_of]
		= Try_Table[tab_idx].stored_move[i];
		++(depth_pre_sort[tab_idx][resultCont].number_of);
	}
}





void first_sort(short tab_idx){
	
	iSquare* opp_piece = NULL, *my_piece = NULL;
	int prom_idx[] = {2, 2, 2, 10, 10, 10, 2, 2}; /* err ? */
	int opp_king = 0, cap_help = NO_CAPTURE, 
		active_piece = 0, captured_piece = 0,
		resCont = 0, 
		opp_rook_count = 0,
		opp_light_count = 0,
		queen_in_danger = FALSE,
		rook_in_danger = FALSE,
		own_light_count = 0,
		light_in_danger = 0;

	iSquare oKingSq = '\0', 
			active_help = INVALID_SQUARE,
			dest_help = INVALID_SQUARE,
			iter_help = INVALID_SQUARE,
			opp_queen = INVALID_SQUARE,
			own_queen = INVALID_SQUARE,
			own_rook = INVALID_SQUARE; /* Attacks on the opponent's queen */
	iSquare own_light[4] = {INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE};
	
	iSquare opp_rook[2] = {INVALID_SQUARE, INVALID_SQUARE};
	iSquare opp_light[4] = {INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE, INVALID_SQUARE};
	char act_color = GET_PIECE_COLOR(Try_Board[tab_idx].extraInfo);

	if(act_color == WHITE){
		my_piece = Try_Board[tab_idx].white_Pieces;
		opp_piece = Try_Board[tab_idx].black_Pieces;
		opp_king = Try_Board[tab_idx].king_b;
		oKingSq = Try_Board[tab_idx].black_Pieces[opp_king];
	}
	else{
		my_piece = Try_Board[tab_idx].black_Pieces;
		opp_piece = Try_Board[tab_idx].white_Pieces;
		opp_king = Try_Board[tab_idx].king_w;
		oKingSq = Try_Board[tab_idx].white_Pieces[opp_king];
	}

	for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){
		iter_help = opp_piece[i];
		if(iter_help != INVALID_SQUARE){
			captured_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(iter_help)]
				[GET_RANK(iter_help)]);			
			if((captured_piece == BISHOP_ID || captured_piece == KNIGHT_ID) &&
				opp_light_count < 4){
				opp_light[opp_light_count] = iter_help;
				++opp_light_count;
			}
			else if(captured_piece == ROOK_ID && opp_rook_count < 2){
				opp_rook[opp_rook_count] = iter_help;
				++opp_rook_count;
			}
			else if(captured_piece == QUEEN_ID)
				opp_queen = iter_help;
			
		}
		iter_help = my_piece[i];
		if(iter_help != INVALID_SQUARE){
			captured_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(iter_help)]
				[GET_RANK(iter_help)]);
				if((captured_piece == KNIGHT_ID || captured_piece == BISHOP_ID)
					&& own_light_count < 4){
						if(quick_check(Try_Board+tab_idx, iter_help)){
							own_light[own_light_count] = iter_help;
							++own_light_count;
							light_in_danger = own_light_count;
						}
				}
				else if(captured_piece == ROOK_ID){
					if(rook_in_danger = quick_check(Try_Board+tab_idx, iter_help))
						own_rook = iter_help;
				}
				else if(captured_piece == QUEEN_ID){
					own_queen = iter_help;
					queen_in_danger = quick_check(Try_Board+tab_idx,own_queen);
				}

		}
	}

	for(int i = 0; i < HYPER_CRITERIA; ++i)
		hyper_cont[i].number_of = 0;	
	
	for(int i = 0; i < Try_Table[tab_idx].number_of; ++i){
		cap_help = Try_Table[tab_idx].stored_move[i].captured_id;
		active_help = Try_Table[tab_idx].stored_move[i].start;
		active_piece = GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(active_help)]
			[GET_RANK(active_help)]);
		resCont = OTHER_MOVE;

		/* First: look_for_check */
		memcpy(&hyper_check, Try_Board+tab_idx, sizeof(situation));
		execute_move(&hyper_check, Try_Table[tab_idx].stored_move+i);
		if(quick_check(&hyper_check, oKingSq))
			resCont = 0;
		else if(Try_Table[tab_idx].stored_move[i].promote)
			resCont = prom_idx[(Try_Table[tab_idx].stored_move[i].promote & 7)];


		else if(cap_help != NO_CAPTURE){
			if(captured_piece = 
				GET_PIECE_TYPE(Try_Board[tab_idx].square[GET_FILE(opp_piece[cap_help])]
							[GET_RANK(opp_piece[cap_help])]))
					resCont = capture_idx[active_piece][captured_piece];
		}
		else{
			dest_help = Try_Table[tab_idx].stored_move[i].dest;
			/* Attack moves */
			if(quick_check(&hyper_check, opp_queen))
					resCont = ATTACK_ON_QUEEN;
			
			else if(queen_in_danger 
				&& active_piece == QUEEN_ID 
				&& !quick_check(Try_Board+tab_idx, dest_help))
					resCont = QUEEN_PROTECT;

			else if(quick_check(&hyper_check, opp_rook[0]) 
				|| quick_check(&hyper_check, opp_rook[1]))
					resCont = ATTACK_ON_ROOK;

			else if(rook_in_danger
				&& active_help == own_rook
				&& !quick_check(Try_Board+tab_idx, dest_help))
					resCont = ROOK_PROTECT;

			else if(opp_light_count){
				for(int k = 0; k < opp_light_count; ++k)
					if(quick_check(&hyper_check, opp_light[k])){
						resCont = ATTACK_ON_LIGHT;
						break;
					}
			}
			else if(light_in_danger){
				if(active_help == own_light[light_in_danger-1]
				  && !quick_check(Try_Board+tab_idx, dest_help))
					  resCont = LIGHT_PROTECT;
			}

			else Try_Table[tab_idx].stored_move[i].quality = evaluate(&hyper_check);
		}		

		hyper_cont[resCont].stored_move[hyper_cont[resCont].number_of] =
			Try_Table[tab_idx].stored_move[i];
		hyper_cont[resCont].number_of = hyper_cont[resCont].number_of + 1;

	} /* END OF for-loop */
	if(hyper_cont[OTHER_MOVE].number_of){
		if(act_color == WHITE)
			qsort(hyper_cont[OTHER_MOVE].stored_move, hyper_cont[OTHER_MOVE].number_of,
				  sizeof(move), white_sort);
		else
			qsort(hyper_cont[OTHER_MOVE].stored_move, hyper_cont[OTHER_MOVE].number_of,
					sizeof(move), black_sort);
	}
				
	int helper = 0;
	for(int i = 0; i < Try_Table[tab_idx].number_of;){
		for(int j = 0; j < hyper_cont[helper].number_of; ++j){
			Try_Table[tab_idx].stored_move[i] =
				hyper_cont[helper].stored_move[j];
			++i;
		}
		++helper;
	}
}
/* -----------------------------------------
		 END OF SORTING FUNCTIONS
   -----------------------------------------*/



/* ---------------------------------------------------
		MOVE GENERATION FUNCTIONS AND THEIR HELPERS
   --------------------------------------------------- */

/* A very rare imaginable case, when capturing en passant is impossible due to a pin,
   is treated with this function.
   I doubt such a position will ever happen, but for correctness' sake ... */
int en_passant_extra(situation* tested, iSquare active){
	
	iSquare cap_pawn = {'\0'};

	if(GET_RANK((*tested).epSquare) == 2)
		cap_pawn = index_convert(GET_FILE((*tested).epSquare), 3);
	else
		cap_pawn = index_convert(GET_FILE((*tested).epSquare), 4);

	char a_color = GET_PIECE_COLOR((*tested).extraInfo);


	iSquare k_square = (a_color == WHITE ? ((*tested).white_Pieces[(*tested).king_w])
		: ((*tested).black_Pieces[(*tested).king_b]));

	if(GET_RANK(k_square) != GET_RANK(cap_pawn))
		return FALSE;

	short c = (GET_FILE(k_square) < GET_FILE(cap_pawn) ?
		1 : -1);

	iSquare helper = '\0';
	char duo_found = FALSE;
	
	for(int i = (GET_FILE(k_square))+ c; i >=0 && i <= NUMBER_OF_FILES-1; i+=c){
		helper = index_convert(i,GET_RANK(k_square));
		
		if(GET_PIECE_COLOR((*tested).square[i][GET_RANK(k_square)])){
			if(!duo_found && helper != active && helper != cap_pawn)
				return FALSE;
			else if(!duo_found && (helper == active || helper == cap_pawn))
				duo_found = TRUE;
			else if(duo_found && !(helper == active || helper == cap_pawn)){
				if(GET_PIECE_TYPE((*tested).square[i][GET_RANK(helper)])
					!= ROOK_ID && GET_PIECE_TYPE((*tested).square[i][GET_RANK(helper)])
					!= QUEEN_ID)
					return FALSE;
				else{
					if(GET_PIECE_COLOR((*tested).square[i][GET_RANK(helper)]) !=
						a_color)
						return TRUE;
					else
						return FALSE;
				}
			}
		}
	}
			
return FALSE;
}

	


/* this could possibly help to speed up is_under_check */
static Destination* i_u_c = NULL;


int quick_check(situation* examined, iSquare xy){
	char side_to_move = GET_PIECE_COLOR((*examined).extraInfo);
	char opposite_site =
		(side_to_move == WHITE ? BLACK : WHITE);

	iSquare helper = {'\0'};
	char detected = {'\0'};
	short moves_per_dir = 0;
	
	i_u_c = fixed_Destination[QUEEN_ID-1][GET_FILE(xy)][GET_RANK(xy)];

	/* queen_test */
	for(long i = SOUTHWEST; i <= SOUTH; ++i){
		moves_per_dir = i_u_c[i].number_of;
		if(moves_per_dir){
			for(long j=0; j < moves_per_dir; ++j){
				helper = i_u_c[i].dest_Square[j];
				detected = (*examined).square[GET_FILE(helper)][GET_RANK(helper)];
				if(GET_PIECE_COLOR(detected) == opposite_site){

					if(GET_PIECE_TYPE(detected) == PAWN_ID){
						if(!j){
						if(side_to_move == WHITE && (i==NORTHWEST || i==NORTHEAST))
							return TRUE;
						
					
						else if(side_to_move == BLACK && (i==SOUTHWEST || i==SOUTHEAST))
							return TRUE;						
						
						else
							break;
						}
						else
							break;
					}



					if(GET_PIECE_TYPE(detected) == QUEEN_ID)
						return TRUE;
					
						
					if(GET_PIECE_TYPE(detected) == ROOK_ID){
						if(i == SOUTH || i == NORTH || i == WEST || i == EAST)
							return TRUE;
						else 
							break;
					}

					if(GET_PIECE_TYPE(detected) == BISHOP_ID){
						if(i == SOUTHWEST || i == NORTHWEST || i == NORTHEAST ||
							i == SOUTHEAST)
							return TRUE;
						else
							break;
					}
							
					/* this is only used to determine whether a square is
									controled by the opposite king */
					if(GET_PIECE_TYPE(detected) == KING_ID &&
						j == 0)	
							return TRUE;

					else /* could only be a knight */
						break;
					
				}
				else if(GET_PIECE_COLOR(detected) == side_to_move)
					break;
			}
		}
	}

	i_u_c = fixed_Destination[KNIGHT_ID-1][GET_FILE(xy)][GET_RANK(xy)];
	/* KNIGHT_TEST */
	for(int i=0; i < i_u_c[STANDARD_DIR].number_of; ++i){
		helper = i_u_c[STANDARD_DIR].dest_Square[i];
		detected = (*examined).square[GET_FILE(helper)][GET_RANK(helper)];
		if(GET_PIECE_COLOR(detected) == opposite_site &&
			GET_PIECE_TYPE(detected) == KNIGHT_ID)
			return TRUE;
	}

	return FALSE;
}



long is_under_check(situation* examined, iSquare xy){
	char side_to_move = GET_PIECE_COLOR((*examined).extraInfo);
	char opposite_site =
		(side_to_move == WHITE ? BLACK : WHITE);

	iSquare helper = {'\0'};
	char detected = {'\0'};
	short moves_per_dir = 0;
	
	long check_counter = 0;
	long help_c_counter = 0;

	i_u_c = fixed_Destination[QUEEN_ID-1][GET_FILE(xy)][GET_RANK(xy)];

	/* queen_test */
	for(long i = SOUTHWEST; i <= SOUTH; ++i){
		moves_per_dir = i_u_c[i].number_of;
		if(moves_per_dir){
			for(long j=0; j < moves_per_dir; ++j){
				helper = i_u_c[i].dest_Square[j];
				detected = (*examined).square[GET_FILE(helper)][GET_RANK(helper)];
				if(GET_PIECE_COLOR(detected) == opposite_site){

					if(GET_PIECE_TYPE(detected) == PAWN_ID){
						if(!j){
						if(side_to_move == WHITE && (i==NORTHWEST || i==NORTHEAST)){
							if(!check_counter){
								check_counter |= helper;
								check_counter <<=8;
							}
							else if((check_counter & CHECK_COUNTER_MASK) == 1){
							help_c_counter |= helper;
							help_c_counter <<= 20; /* only the source, not the direction must be stored */
							check_counter |= help_c_counter;
							help_c_counter = 0;

							}
						++check_counter;
						break;
						}
					
						else if(side_to_move == BLACK && (i==SOUTHWEST || i==SOUTHEAST)){
							if(!check_counter){
								check_counter |= helper;
								check_counter <<=8;
							}
							else if((check_counter & CHECK_COUNTER_MASK) == 1){
							help_c_counter |= helper;
							help_c_counter <<= 20; /* only the source, not the direction must be stored */
							check_counter |= help_c_counter;
							help_c_counter = 0;

							}
						++check_counter;
						break;
						}
						
						else
							break;
						}
						else
							break;
					}



					if(GET_PIECE_TYPE(detected) == QUEEN_ID){
						if(!check_counter){
							check_counter |= helper;
							check_counter <<=8;
							check_counter = check_counter | (i << 4);
						}
						else if((check_counter & CHECK_COUNTER_MASK) == 1){
							help_c_counter |= helper;
							help_c_counter <<= 20;
							help_c_counter |= (i << 16); /* for the case of a double_check */
							check_counter |= help_c_counter;
							help_c_counter = 0;
						}

						++check_counter;
						break;
					}
						
					if(GET_PIECE_TYPE(detected) == ROOK_ID &&
						(i == SOUTH || i == NORTH || i == WEST || i == EAST)){
							if(!check_counter){
								check_counter |= helper;
								check_counter <<=8;
								check_counter = check_counter | (i << 4);
							}

							else if((check_counter & CHECK_COUNTER_MASK) == 1){
							help_c_counter |= helper;
							help_c_counter <<= 20;
							help_c_counter |= (i << 16); /* for the case of a double_check */
							check_counter |= help_c_counter;
							help_c_counter = 0;
							}

						++check_counter;
						break;
						}

					else if(GET_PIECE_TYPE(detected) == ROOK_ID &&
						!(i == SOUTH || i == NORTH || i == WEST || i == EAST))
						break;

					if(GET_PIECE_TYPE(detected) == BISHOP_ID &&
						(i == SOUTHWEST || i == NORTHWEST || i == NORTHEAST ||
						i == SOUTHEAST)){
							if(!check_counter){
								check_counter |= helper;
								check_counter <<=8;
								check_counter = check_counter | (i << 4);
							}
							else if((check_counter & CHECK_COUNTER_MASK) == 1){
							help_c_counter |= helper;
							help_c_counter <<= 20;
							help_c_counter |= (i << 16); /* for the case of a double_check */
							check_counter |= help_c_counter;
							help_c_counter = 0;
						}
						++check_counter;
						break;
						}

					else if(GET_PIECE_TYPE(detected) == BISHOP_ID &&
						!(i == SOUTHWEST || i == NORTHWEST || i == NORTHEAST ||
						i == SOUTHEAST))
						break;

					if(GET_PIECE_TYPE(detected) == KING_ID &&
						j == 0){	/* this is only used to determine whether a square is
									controled by the opposite king */
							if(!check_counter){
								check_counter |= helper;
								check_counter <<=8;
								check_counter = check_counter | (i << 4);
							}
						++check_counter;
						break;
						}

					

					else
						break;
					
				}
				else if(GET_PIECE_COLOR(detected) == side_to_move)
					break;
			}
		}
	}


	/* KNIGHT_TEST */
	for(int i=0; i < fixed_Destination[KNIGHT_ID-1][GET_FILE(xy)][GET_RANK(xy)][STANDARD_DIR].number_of; i++){
		helper = fixed_Destination[KNIGHT_ID-1][GET_FILE(xy)][GET_RANK(xy)][STANDARD_DIR].dest_Square[i];
		detected = (*examined).square[GET_FILE(helper)][GET_RANK(helper)];
		if(GET_PIECE_COLOR(detected) == opposite_site &&
			GET_PIECE_TYPE(detected) == KNIGHT_ID){
				if(!check_counter){
					check_counter |= helper;
					check_counter <<=8;
					}
				else if((check_counter & CHECK_COUNTER_MASK) == 1){
					help_c_counter |= helper;
					help_c_counter <<= 20; /* only the source, not the direction must be stored */
					check_counter |= help_c_counter;
					help_c_counter = 0;

							}
			++check_counter;
			break;
			}
	}
	return check_counter;
}

/* Disable Move directions due to pins */
int lock_directions(situation* examined, iSquare piece_loc, short* lock_indicator){
	char side_to_move = GET_PIECE_COLOR((*examined).extraInfo);
	char opposite_site = '\0';
	char act_king = '\0';
	iSquare* checkedField = NULL;

	

	if(side_to_move == WHITE){
		opposite_site = BLACK;
		act_king = (*examined).king_w;
		checkedField = (*examined).white_Pieces;
	}
	else{
		opposite_site = WHITE;
		act_king = (*examined).king_b;
		checkedField = (*examined).black_Pieces;
	}

	char piece_rank = GET_RANK(piece_loc),
		piece_file = GET_FILE(piece_loc),
		king_rank = GET_RANK(checkedField[act_king]),
		king_file = GET_FILE(checkedField[act_king]);


	/* in these cases it is impossible for a piece to be bound */
	if(abs(piece_rank - king_rank) !=
		abs(piece_file - king_file) &&
		piece_rank != king_rank &&
		piece_file != king_file)
		return FALSE;

	short dion = 0;

	/* determine which direction to search */
	if(king_file == piece_file)
		king_rank < piece_rank ? (dion = NORTH) : (dion = SOUTH);
	else if(king_rank == piece_rank)
		king_file < piece_file ? (dion = EAST) : (dion = WEST);
	else if(king_rank < piece_rank)
		piece_file < king_file ? (dion = NORTHWEST) : (dion = NORTHEAST);
	else
		piece_file < king_file ? (dion = SOUTHWEST) : (dion = SOUTHEAST);

	/* QUEEN_TEST */

	i_u_c = fixed_Destination[QUEEN_ID-1][king_file][king_rank];

	short moves_per_dir = i_u_c[dion].number_of;

	iSquare helper = {'\0'};
	iSquare detected = {'\0'};

	iSquare helper_b = {'\0'};
	iSquare detected_b = {'\0'};

	for(int j = 0; j < moves_per_dir; ++j){
			helper = i_u_c[dion].dest_Square[j]; /* stores a square */

			detected = (*examined).square[GET_FILE(helper)][GET_RANK(helper)];

			if(GET_PIECE_COLOR(detected) == side_to_move &&
				helper == piece_loc){
					
					for(int k = j+1; k < moves_per_dir; ++k){
						helper_b = i_u_c[dion].dest_Square[k];
						
						detected_b = (*examined).square[GET_FILE(helper_b)][GET_RANK(helper_b)];
						
						if(GET_PIECE_COLOR(detected_b) == opposite_site){
							
							if(GET_PIECE_TYPE(detected_b) == QUEEN_ID ||
								GET_PIECE_TYPE(detected_b) == ROOK_ID){
									if(dion == NORTH || dion == SOUTH){
										for(int t = STANDARD_DIR; t <= SOUTH; ++t)
											if(t != NORTH && t != SOUTH)
												lock_indicator[t] = TRUE;
										break;
									}
									else if(dion == WEST || dion == EAST){
										for(int t = STANDARD_DIR; t <= SOUTH; ++t)
											if(t != WEST && t != EAST)
												lock_indicator[t] = TRUE;
										break;
									}
									else if(dion == SOUTHWEST || dion == NORTHEAST){
										if(GET_PIECE_TYPE(detected_b) != ROOK_ID){
											for(int t = STANDARD_DIR; t <= SOUTH; ++t)
												if(t != SOUTHWEST && t != NORTHEAST)
													lock_indicator[t] = TRUE;
										}
										break;
									}
									else if(dion == NORTHWEST || dion == SOUTHEAST){
										if(GET_PIECE_TYPE(detected_b) != ROOK_ID){
											for(int t = STANDARD_DIR; t <= SOUTH; ++t)
												if(t != NORTHWEST && t != SOUTHEAST)
													lock_indicator[t] = TRUE;
											}
										break;
									}
											
								}

						
							if(GET_PIECE_TYPE(detected_b) == BISHOP_ID){
								if(dion == NORTHWEST || dion == SOUTHEAST){
									for(int t = STANDARD_DIR; t <= SOUTH; ++t)
										if(t != NORTHWEST && t != SOUTHEAST)
											lock_indicator[t] = TRUE;
									break;
								}
								else if(dion == SOUTHWEST || dion == NORTHEAST){
									for(int t = STANDARD_DIR; t <= SOUTH; ++t)
										if(t != SOUTHWEST && t != NORTHEAST)
											lock_indicator[t] = TRUE;
									break;
								}
								break;
							}

							break;
						}		
						else if(GET_PIECE_COLOR(detected_b) == side_to_move)
							break;
					}
				}
			else if(GET_PIECE_COLOR(detected) == opposite_site ||
				(GET_PIECE_COLOR(detected) == side_to_move && helper != piece_loc))
				break;
	}
	
	return TRUE;

}



/* this is a simplified find_moves function, that is used by the eval function */
int find_quickly(situation* examined, move_table* results){
	memset(results,'\0',sizeof(move_table));
	
	char A_color = GET_PIECE_COLOR((*examined).extraInfo);
	iSquare* side_to_move = NULL, *opposite_site = NULL;
	short k_index = 0;

	if(A_color == WHITE){
		side_to_move = &((*examined).white_Pieces[0]);
		opposite_site = &((*examined).black_Pieces[0]);
		k_index = (*examined).king_w;
	}
	else{
		side_to_move = &((*examined).black_Pieces[0]);
		opposite_site = &((*examined).white_Pieces[0]);
		k_index = (*examined).king_b;
	}
	
	/* these numbers are used to determine whether the active king is under check */
	long c_counter = 0;
	iSquare c_source_1 = '\0', c_source_2 = '\0';
	iSquare c_dir_1 = '\0', c_dir_2 = '\0';

	c_counter = is_under_check(examined,side_to_move[k_index]);
	c_source_1 = (c_counter >> 8) & CHECK_SOURCE_MASK;
	c_source_2 = (c_counter >> 20) & CHECK_SOURCE_MASK;
	c_dir_1 = (c_counter >> 4) & CHECK_COUNTER_MASK;
	c_dir_2 = (c_counter >> 16) & CHECK_COUNTER_MASK;

	int piece_type_index = 0, file = 0, rank = 0, ch_file = 0, ch_rank = 0;
	int moves_of_direction = 0;
	iSquare checked_square = '\0';
	iSquare helper = '\0';

	short Forbidden_dir[9] = {0};
	short inner_loop_breaker = FALSE;
	char col_helper = '\0';
	
	
	/* why ? */
	move exam_move = {'\0', '\0', '\0', '\0', '\0', 0};
	char miniBuffer[4] = { '\0'};	
	
	for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){

		if(side_to_move[i] != INVALID_SQUARE){

			memset(Forbidden_dir,'\0',NUMBER_OF_DIRECTIONS*sizeof(short));

			file = GET_FILE(side_to_move[i]);
			rank = GET_RANK(side_to_move[i]);

			piece_type_index = GET_PIECE_TYPE((*examined).square[file][rank]) - 1;
			
			if(piece_type_index < 0 || piece_type_index > PAWN_ID-1 ||
				file < 0 || file > 7 || rank < 0 || rank > 7)
					return FALSE;

			/* lock directions of Pawn moves based on the active color */
			
			if(piece_type_index == PAWN_ID-1 && A_color == BLACK){

				Forbidden_dir[NORTH] = 1;
				Forbidden_dir[NORTHEAST] = 1;
				Forbidden_dir[NORTHWEST] = 1;
				
				if(fixed_Destination[piece_type_index][file][rank][SOUTHWEST].number_of){
					helper = add_vector(index_convert(file,rank),-1,-1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= WHITE && helper != (*examined).epSquare)
						Forbidden_dir[SOUTHWEST] = 1;
				}

				if(fixed_Destination[piece_type_index][file][rank][SOUTHEAST].number_of){
					helper = add_vector(index_convert(file,rank),1,-1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= WHITE && helper != (*examined).epSquare)
						Forbidden_dir[SOUTHEAST] = 1;
				}
			}

			else if(piece_type_index == PAWN_ID-1 && A_color == WHITE){
					
				Forbidden_dir[SOUTH] = 1;
				Forbidden_dir[SOUTHWEST] = 1;
				Forbidden_dir[SOUTHEAST] = 1;



				if(fixed_Destination[piece_type_index][file][rank][NORTHWEST].number_of){
					helper = add_vector(index_convert(file,rank),-1,1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= BLACK && helper != (*examined).epSquare)
						Forbidden_dir[NORTHWEST] = 1;
				}

			
				if(fixed_Destination[piece_type_index][file][rank][NORTHEAST].number_of){
					helper = add_vector(index_convert(file,rank),1,1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= BLACK && helper != (*examined).epSquare)
						Forbidden_dir[NORTHEAST] = 1;
				}
			}
				
			/* determines, which pieces are bound */
			if(piece_type_index != KING_ID-1)
				lock_directions(examined,index_convert(file,rank),Forbidden_dir);	

			for(int j = STANDARD_DIR; j <= SOUTH; ++j){
				
				moves_of_direction =
					fixed_Destination[piece_type_index][file][rank][j].number_of;

				if(moves_of_direction && !(Forbidden_dir[j])){
					
					/* the inner loop_breaker will be true, once a capturing move is encountered */
					
					for(int k=0; (k < moves_of_direction &&
						!inner_loop_breaker); ++k){
						
						if(j < STANDARD_DIR || j > SOUTH)
								return false;	/* DELETED LOG FILE */
						
						checked_square =
							fixed_Destination[piece_type_index][file][rank][j].dest_Square[k];
						ch_file = GET_FILE(checked_square);
						ch_rank = GET_RANK(checked_square);
						
						/* if the king is under double_check only the King can move */
						if((c_counter & CHECK_COUNTER_MASK) > 1 && piece_type_index != KING_ID-1)
							break;
												
						/* only the Knight's STANDARD_DIR - Loop does not break
						   as soon as we encounter an own piece on a destination square */
						if(GET_PIECE_COLOR((*examined).square[ch_file][ch_rank]) == A_color){
							if(piece_type_index != KNIGHT_ID-1)
								break;
							else
								continue;
						}
						
						/* a pawn can't capture to the NORTH or to the SOUTH */
						if(GET_PIECE_COLOR((*examined).square[ch_file][ch_rank])  &&
							piece_type_index == PAWN_ID-1 && (j == NORTH || j == SOUTH))
							break;
						
						
						/* A king must not move on a square, that is attacked */
						if(piece_type_index == KING_ID-1){
							if(quick_check(examined,checked_square))
								break;
							
							if(c_counter){
								if(c_dir_1 && (j == c_dir_1 + 4 || j + 4 == c_dir_1))
									break;
								
								if(c_dir_2 && (j == c_dir_2 + 4 || j + 4 == c_dir_2))
									break;

							}

							/* this means we are examining a castling move */
							if(k){
								if(A_color == BLACK){
									if(rank == NUMBER_OF_RANKS-1){
										if(j == WEST){
											if(!((*examined).extraInfo & BQCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file-1][rank])
												|| GET_PIECE_COLOR((*examined).square[file-2][rank])
												|| GET_PIECE_COLOR((*examined).square[file-3][rank]))
												break;
										}
										if(j == EAST){
											if(!((*examined).extraInfo & BKCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file+1][rank])
												|| GET_PIECE_COLOR((*examined).square[file+2][rank]))
												break;
										}
									}
									else
										break;
								}
								else{
									if(!rank){
										if(j == WEST){
											if(!((*examined).extraInfo & WQCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file-1][rank])
												|| GET_PIECE_COLOR((*examined).square[file-2][rank])
												|| GET_PIECE_COLOR((*examined).square[file-3][rank]))
												break;
										}
										if(j == EAST){
											if(!((*examined).extraInfo & WKCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file+1][rank])
												|| GET_PIECE_COLOR((*examined).square[file+2][rank]))
												break;
										}
									}
									else
										break;
								}
							}
							
						}

							if(GET_PIECE_COLOR((*examined).square[ch_file][ch_rank])
								&& piece_type_index != KNIGHT_ID-1)
								inner_loop_breaker = TRUE;							

							if(c_counter && piece_type_index != KING_ID-1 && c_dir_1){
								helper = FALSE;

								for(int p = 0;
									p < fixed_Destination[QUEEN_ID-1][GET_FILE(side_to_move[k_index])]
									[GET_RANK(side_to_move[k_index])][c_dir_1].number_of; ++p){

										if((fixed_Destination[QUEEN_ID-1][GET_FILE(side_to_move[k_index])]
										[GET_RANK(side_to_move[k_index])][c_dir_1].dest_Square[p]) ==
											c_source_1){
												if(checked_square == c_source_1)
													helper = TRUE;
												break;
											}

										else{
											if((fixed_Destination[QUEEN_ID-1][GET_FILE(side_to_move[k_index])]
										[GET_RANK(side_to_move[k_index])][c_dir_1].dest_Square[p]) ==
											checked_square)
												helper = TRUE;
										}
									}

									if(!helper)
										continue;

							}

							else if(c_counter && piece_type_index != KING_ID-1 && !c_dir_1
								&& checked_square != c_source_1){
									if(piece_type_index != PAWN_ID-1)
										continue;
									if(GET_PIECE_TYPE((*examined).square[GET_FILE(c_source_1)]
										[GET_RANK(c_source_1)]) == PAWN_ID &&
											checked_square != (*examined).epSquare &&
											checked_square != c_source_1)
											continue;
									if(GET_PIECE_TYPE((*examined).square[GET_FILE(c_source_1)]
										[GET_RANK(c_source_1)]) == KNIGHT_ID)
											continue;
								}
	

							if(checked_square == (*examined).epSquare &&
								piece_type_index == PAWN_ID-1)
								if(en_passant_extra(examined,side_to_move[i]))
									break;		
						
						/* exam_move: if the loop is not yet broken,
						   this means there is at least one possible move */
							(*results).number_of = 1;
							if(c_counter)
								(*results).check = TRUE;
							else
								(*results).check = FALSE;
							(*results).mate = FALSE;
							return TRUE;

						} /* k loop _ */
				inner_loop_breaker = FALSE;
				}
			}
			
		}
		
	} /* for i < NUMBER OF PIECES */

if(!(*results).number_of && c_counter)
		(*results).mate = TRUE;

if(c_counter)
	(*results).check = TRUE;
else
(*results).check = FALSE;

return 0;
}


/* This is the very important function to find all the moves which are possible
   in a given situation. The function's results are stored to its parameter move_table 'results' */
int find_moves(situation* examined, move_table* results){
	
	memset(results,'\0',sizeof(move_table));
	
	char A_color = GET_PIECE_COLOR((*examined).extraInfo);
	iSquare* side_to_move = NULL, *opposite_site = NULL;
	short k_index = 0;

	if(A_color == WHITE){
		side_to_move = &((*examined).white_Pieces[0]);
		opposite_site = &((*examined).black_Pieces[0]);
		k_index = (*examined).king_w;
	}
	else{
		side_to_move = &((*examined).black_Pieces[0]);
		opposite_site = &((*examined).white_Pieces[0]);
		k_index = (*examined).king_b;
	}

	/* these numbers are used to determine whether the active king is under check */
	long c_counter = 0;
	iSquare c_source_1 = '\0', c_source_2 = '\0';
	iSquare c_dir_1 = '\0', c_dir_2 = '\0';

	c_counter = is_under_check(examined,side_to_move[k_index]);
	c_source_1 = (c_counter >> 8) & CHECK_SOURCE_MASK;
	c_source_2 = (c_counter >> 20) & CHECK_SOURCE_MASK;
	c_dir_1 = (c_counter >> 4) & CHECK_COUNTER_MASK;
	c_dir_2 = (c_counter >> 16) & CHECK_COUNTER_MASK;

	int piece_type_index = 0, file = 0, rank = 0, ch_file = 0, ch_rank = 0;
	int moves_of_direction = 0;
	iSquare checked_square = '\0';
	iSquare helper = '\0';

	short Forbidden_dir[9] = {0};
	short inner_loop_breaker = FALSE;
	
	char col_helper = '\0';	
	
	/* why ? */
	move exam_move = {'\0', '\0', '\0', '\0', '\0', 0};
	char miniBuffer[4] = { '\0'};	
	
	for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){

		if(side_to_move[i] != INVALID_SQUARE){

			memset(Forbidden_dir,'\0',NUMBER_OF_DIRECTIONS*sizeof(short));

			file = GET_FILE(side_to_move[i]);
			rank = GET_RANK(side_to_move[i]);

			piece_type_index = GET_PIECE_TYPE((*examined).square[file][rank]) - 1;
			
			if(piece_type_index < 0 || piece_type_index > PAWN_ID-1 ||
				file < 0 || file > 7 || rank < 0 || rank > 7)
					/* DELETED LOG_FILE */
					return FALSE;
				


			/* lock directions of Pawn moves based on the active color */
			
			if(piece_type_index == PAWN_ID-1 && A_color == BLACK){

				Forbidden_dir[NORTH] = 1;
				Forbidden_dir[NORTHEAST] = 1;
				Forbidden_dir[NORTHWEST] = 1;
				
				if(fixed_Destination[piece_type_index][file][rank][SOUTHWEST].number_of){
					helper = add_vector(index_convert(file,rank),-1,-1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= WHITE && helper != (*examined).epSquare)
						Forbidden_dir[SOUTHWEST] = 1;
				}

				if(fixed_Destination[piece_type_index][file][rank][SOUTHEAST].number_of){
					helper = add_vector(index_convert(file,rank),1,-1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= WHITE && helper != (*examined).epSquare)
						Forbidden_dir[SOUTHEAST] = 1;
				}
			}

			else if(piece_type_index == PAWN_ID-1 && A_color == WHITE){
					
				Forbidden_dir[SOUTH] = 1;
				Forbidden_dir[SOUTHWEST] = 1;
				Forbidden_dir[SOUTHEAST] = 1;



				if(fixed_Destination[piece_type_index][file][rank][NORTHWEST].number_of){
					helper = add_vector(index_convert(file,rank),-1,1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= BLACK && helper != (*examined).epSquare)
						Forbidden_dir[NORTHWEST] = 1;
				}

			
				if(fixed_Destination[piece_type_index][file][rank][NORTHEAST].number_of){
					helper = add_vector(index_convert(file,rank),1,1);
					if(GET_PIECE_COLOR((*examined).square[GET_FILE(helper)][GET_RANK(helper)])
						!= BLACK && helper != (*examined).epSquare)
						Forbidden_dir[NORTHEAST] = 1;
				}
			}
				
			/* determines, which pieces are bound */
			if(piece_type_index != KING_ID-1)
				lock_directions(examined,index_convert(file,rank),Forbidden_dir);

			for(int j = STANDARD_DIR; j <= SOUTH; ++j){
				
				moves_of_direction =
					fixed_Destination[piece_type_index][file][rank][j].number_of;

				if(moves_of_direction && !(Forbidden_dir[j])){
					
					/* the inner loop_breaker will be true, once a capturing move is encountered */
					
					for(int k=0; (k < moves_of_direction &&
						!inner_loop_breaker); ++k){
						
						if(j < STANDARD_DIR || j > SOUTH)
								return false; /* DELETED LOGFILE */
						
						checked_square =
							fixed_Destination[piece_type_index][file][rank][j].dest_Square[k];
						ch_file = GET_FILE(checked_square);
						ch_rank = GET_RANK(checked_square);
						
						/* if the king is under double_check only the King can move */
						if((c_counter & CHECK_COUNTER_MASK) > 1 && piece_type_index != KING_ID-1)
							break;
					
						/* only the Knight's STANDARD_DIR - Loop does not break
						   as soon as we encounter an own piece on a destination square */
						if(GET_PIECE_COLOR((*examined).square[ch_file][ch_rank]) == A_color){
							if(piece_type_index != KNIGHT_ID-1)
								break;
							else
								continue;
						}
						
						/* a pawn can't capture to the NORTH or SOUTH */
						if(GET_PIECE_COLOR((*examined).square[ch_file][ch_rank])  &&
							piece_type_index == PAWN_ID-1 && (j == NORTH || j == SOUTH))
							break;
					
						/* A king must not move on a square, which is attacked */
						if(piece_type_index == KING_ID-1){
							if(quick_check(examined,checked_square))
								break;
							
							if(c_counter){
								if(c_dir_1 && (j == c_dir_1 + 4 || j + 4 == c_dir_1))
									break;
								
								if(c_dir_2 && (j == c_dir_2 + 4 || j + 4 == c_dir_2))
									break;

							}

							/* this means we are examining a castling move */
							if(k){
								if(A_color == BLACK){
									if(rank == NUMBER_OF_RANKS-1){
										if(j == WEST){
											if(!((*examined).extraInfo & BQCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file-1][rank])
												|| GET_PIECE_COLOR((*examined).square[file-2][rank])
												|| GET_PIECE_COLOR((*examined).square[file-3][rank]))
												break;
										}
										if(j == EAST){
											if(!((*examined).extraInfo & BKCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file+1][rank])
												|| GET_PIECE_COLOR((*examined).square[file+2][rank]))
												break;
										}
									}
									else
										break;
								}
								else{
									if(!rank){
										if(j == WEST){
											if(!((*examined).extraInfo & WQCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file-1][rank])
												|| GET_PIECE_COLOR((*examined).square[file-2][rank])
												|| GET_PIECE_COLOR((*examined).square[file-3][rank]))
												break;
										}
										if(j == EAST){
											if(!((*examined).extraInfo & WKCASTLE) || c_counter)
												break;
											if(GET_PIECE_COLOR((*examined).square[file+1][rank])
												|| GET_PIECE_COLOR((*examined).square[file+2][rank]))
												break;
										}
									}
									else
										break;
								}
							}
							
						}
							
							/* when capturing a piece, this is often the last possible
							   move to that direction. In these cases the inner_loop_breaker
							   is set to TRUE, in order to break the loop
							   after the move has been stored into the move_table */
							if(GET_PIECE_COLOR((*examined).square[ch_file][ch_rank])
								&& piece_type_index != KNIGHT_ID-1)
								inner_loop_breaker = TRUE;							

							if(c_counter && piece_type_index != KING_ID-1 && c_dir_1){
								helper = FALSE;

								for(int p = 0;
									p < fixed_Destination[QUEEN_ID-1][GET_FILE(side_to_move[k_index])]
									[GET_RANK(side_to_move[k_index])][c_dir_1].number_of; ++p){

										if((fixed_Destination[QUEEN_ID-1][GET_FILE(side_to_move[k_index])]
										[GET_RANK(side_to_move[k_index])][c_dir_1].dest_Square[p]) ==
											c_source_1){
												if(checked_square == c_source_1)
													helper = TRUE;
												break;
											}

										else{
											if((fixed_Destination[QUEEN_ID-1][GET_FILE(side_to_move[k_index])]
										[GET_RANK(side_to_move[k_index])][c_dir_1].dest_Square[p]) ==
											checked_square)
												helper = TRUE;
										}
									}

									if(!helper)
										continue;

							}

							else if(c_counter && piece_type_index != KING_ID-1 && !c_dir_1
								&& checked_square != c_source_1){
									if(piece_type_index != PAWN_ID-1)
										continue;
									if(GET_PIECE_TYPE((*examined).square[GET_FILE(c_source_1)]
										[GET_RANK(c_source_1)]) == PAWN_ID &&
											checked_square != (*examined).epSquare &&
											checked_square != c_source_1)
											continue;
									if(GET_PIECE_TYPE((*examined).square[GET_FILE(c_source_1)]
										[GET_RANK(c_source_1)]) == KNIGHT_ID)
											continue;
								}								

							if(checked_square == (*examined).epSquare &&
								piece_type_index == PAWN_ID-1)
								if(en_passant_extra(examined,side_to_move[i]))
									break;			
						
						/* exam_move: if the loop is not yet broken,
						   this means the move is possible */							
							if((*results).number_of < MOVE_TABLE_DIMENSION){
										(*results).stored_move[(*results).number_of].start = 
											side_to_move[i];
										(*results).stored_move[(*results).number_of].dest =
											checked_square;
										(*results).stored_move[(*results).number_of].active_id =
											i;
										(*results).stored_move[(*results).number_of].captured_id =
											NO_CAPTURE;

										if((*examined).square[ch_file][ch_rank]){
											for(int p = 0; p < MAXIMUM_NUMBER_OF_PIECES; ++p)
												if(opposite_site[p] != INVALID_SQUARE
													&& opposite_site[p] == checked_square){
														(*results).stored_move[(*results).number_of].captured_id = p;
														break;
													}

										}

										if(piece_type_index == PAWN_ID-1 && 
											checked_square == (*examined).epSquare){
												if(A_color == WHITE)
													(*results).stored_move[(*results).number_of].captured_id = 
																add_vector(checked_square, 0, -1);
												else
													(*results).stored_move[(*results).number_of].captured_id = 
																add_vector(checked_square, 0, 1);


												for(int p = 0; p < MAXIMUM_NUMBER_OF_PIECES; ++p)
													if(opposite_site[p] != INVALID_SQUARE
													&& opposite_site[p] == (*results).stored_move[(*results).number_of].captured_id)
													{
															(*results).stored_move[(*results).number_of].captured_id = p;
															break;
														}
											}											
										
										/* PAWN PROMOTION */
										if(piece_type_index == PAWN_ID-1 && (!ch_rank  ||
											ch_rank == NUMBER_OF_RANKS-1)){
												if((*results).number_of < MOVE_TABLE_DIMENSION - 3){
													for(int p = 0; p < 3; ++p){
														(*results).stored_move[(*results).number_of+p+1] =
															(*results).stored_move[(*results).number_of+p];
														(*results).stored_move[(*results).number_of+p].promote =
															QUEEN_ID+p;
													}
													(*results).stored_move[(*results).number_of+3].promote =
														QUEEN_ID+3;
													(*results).number_of += 4;
													break;
												}
												else
													(*results).stored_move[(*results).number_of].promote =
													QUEEN_ID;
											}

											(*results).number_of++;
		

								}
							
					} /* k loop _ */
				inner_loop_breaker = FALSE;
				}
			}
			
		}
		
	} /* for i < NUMBER OF PIECES */

if(!(*results).number_of && c_counter)
		(*results).mate = TRUE;

if(c_counter)
	(*results).check = TRUE;
else
(*results).check = FALSE;

return 0;
}
/*  ------------------------------------------------
		END OF MOVE GENERATION (AND ITS HELPERS)
	------------------------------------------------*/





/* -----------------------------
		MOVE EXECUTION()
   -----------------------------*/
/* After a call to this function the board is updated */
char execute_move(situation* board, move* xy){
	char a_color = GET_PIECE_COLOR((*board).extraInfo);
	char o_color = (a_color == WHITE ? BLACK : WHITE);

	iSquare* a_id_stack = (a_color == WHITE ? (*board).white_Pieces :
	(*board).black_Pieces);

	iSquare* o_id_stack = (a_color == WHITE ? (*board).black_Pieces :
	(*board).white_Pieces);

	char* start_mask = &((*board).square[GET_FILE((*xy).start)][GET_RANK((*xy).start)]);
	char* dest_mask = &((*board).square[GET_FILE((*xy).dest)][GET_RANK((*xy).dest)]);

	(*board).hMoves += 1;

	/* handle capturing moves */
	if((*xy).captured_id != NO_CAPTURE){
		
		if(GET_PIECE_TYPE(*start_mask) == PAWN_ID &&
			(*xy).dest == (*board).epSquare)
			(*board).square[GET_FILE(o_id_stack[(*xy).captured_id])]
			[GET_RANK(o_id_stack[(*xy).captured_id])] = '\0';

		if(GET_PIECE_TYPE(*dest_mask) == ROOK_ID){
			if(o_id_stack[(*xy).captured_id] == 
				index_convert(0,NUMBER_OF_RANKS-1))
				(*board).extraInfo &= ~BQCASTLE;
			else if(o_id_stack[(*xy).captured_id] == 
				index_convert(NUMBER_OF_FILES-1,NUMBER_OF_RANKS-1))
				(*board).extraInfo &= ~BKCASTLE;
			else if(o_id_stack[(*xy).captured_id] == 
				index_convert(0,0))
				(*board).extraInfo &= ~WQCASTLE;
			else if(o_id_stack[(*xy).captured_id] == 
				index_convert(NUMBER_OF_FILES-1,0))
				(*board).extraInfo &= ~WKCASTLE;
		}


		o_id_stack[(*xy).captured_id] = INVALID_SQUARE;

		(*board).hMoves = '\0';
	}

	(*board).epSquare = INVALID_SQUARE;

	if(GET_PIECE_TYPE(*start_mask) == PAWN_ID){
		
		(*board).hMoves = '\0';

		if(GET_RANK((*xy).dest) - GET_RANK((*xy).start) == -2)
			(*board).epSquare = add_vector((*xy).dest,0,1);
		else if(GET_RANK((*xy).dest) - GET_RANK((*xy).start) == 2)
			(*board).epSquare = add_vector((*xy).dest,0,-1);
		else
			(*board).epSquare = INVALID_SQUARE;

		if(GET_RANK((*xy).dest) == 0)
			*start_mask = (*xy).promote | BLACK;

		if(GET_RANK((*xy).dest) == NUMBER_OF_RANKS-1)
			*start_mask = (*xy).promote | WHITE;

	}

	else if(GET_PIECE_TYPE(*start_mask) == ROOK_ID){

		if((*xy).start == index_convert(NUMBER_OF_FILES-1,0))
			(*board).extraInfo &= ~WKCASTLE;
		else if((*xy).start == index_convert(0,0))
			(*board).extraInfo &= ~WQCASTLE;
		else if((*xy).start == index_convert(NUMBER_OF_FILES-1,NUMBER_OF_RANKS-1))
			(*board).extraInfo &= ~BKCASTLE;
		else if((*xy).start == index_convert(0,NUMBER_OF_RANKS-1))
			(*board).extraInfo &= ~BQCASTLE;

	}

	else if((*xy).start == index_convert(INITIAL_WHITE_KING_FILE,0) &&
		GET_PIECE_TYPE(*start_mask) == KING_ID)
		(*board).extraInfo &= ~(WKCASTLE | WQCASTLE);

	else if((*xy).start == index_convert(INITIAL_BLACK_KING_FILE,NUMBER_OF_RANKS-1) &&
		GET_PIECE_TYPE(*start_mask) == KING_ID)
		(*board).extraInfo &= ~(BKCASTLE | BQCASTLE);
	
	
	/* this section is used to help with castling moves */
	
	iSquare cst_start = INVALID_SQUARE, cst_dest = INVALID_SQUARE;
	short cst_id = 0;

	if((*xy).start == index_convert(INITIAL_WHITE_KING_FILE,0) &&
		GET_PIECE_TYPE(*start_mask) == KING_ID && 
		(*xy).dest == add_vector((*xy).start,2,0)){
			for(int p = 0; p < MAXIMUM_NUMBER_OF_PIECES; ++p)
				if(a_id_stack[p] == index_convert(NUMBER_OF_FILES-1,0)){
					cst_start = a_id_stack[p];
					cst_dest = index_convert(INITIAL_WHITE_KING_FILE+1,0);
					cst_id = p;
					break;
				}
		}

	else if((*xy).start == index_convert(INITIAL_WHITE_KING_FILE,0) &&
		GET_PIECE_TYPE(*start_mask) == KING_ID && 
		(*xy).dest == add_vector((*xy).start,-2,0)){
			for(int p = 0; p < MAXIMUM_NUMBER_OF_PIECES; ++p)
				if(a_id_stack[p] == index_convert(0,0)){
					cst_start = a_id_stack[p];
					cst_dest = index_convert(INITIAL_WHITE_KING_FILE-1,0);
					cst_id = p;
					break;
				}
		}

	else if((*xy).start == index_convert(INITIAL_BLACK_KING_FILE,NUMBER_OF_RANKS-1) &&
		GET_PIECE_TYPE(*start_mask) == KING_ID && 
		(*xy).dest == add_vector((*xy).start,2,0)){
			for(int p = 0; p < MAXIMUM_NUMBER_OF_PIECES; ++p)
				if(a_id_stack[p] == index_convert(NUMBER_OF_FILES-1,NUMBER_OF_RANKS-1)){
					cst_start = a_id_stack[p];
					cst_dest = index_convert(INITIAL_BLACK_KING_FILE+1,NUMBER_OF_RANKS-1);
					cst_id = p;
					break;
				}
		}

	else if((*xy).start == index_convert(INITIAL_BLACK_KING_FILE,NUMBER_OF_RANKS-1) &&
		GET_PIECE_TYPE(*start_mask) == KING_ID && 
		(*xy).dest == add_vector((*xy).start,-2,0)){
			for(int p = 0; p < MAXIMUM_NUMBER_OF_PIECES; ++p)
				if(a_id_stack[p] == index_convert(0,NUMBER_OF_RANKS-1)){
					cst_start = a_id_stack[p];
					cst_dest = index_convert(INITIAL_BLACK_KING_FILE-1,NUMBER_OF_RANKS-1);
					cst_id = p;
					break;
				}
		}

		/*  End of castling preparation */


		a_id_stack[(*xy).active_id] = (*xy).dest;
		*dest_mask = *start_mask;
		*start_mask = '\0';

		if(cst_start != INVALID_SQUARE){
			a_id_stack[cst_id] = cst_dest;
			(*board).square[GET_FILE(cst_dest)][GET_RANK(cst_dest)] =
				(*board).square[GET_FILE(cst_start)][GET_RANK(cst_start)];
			(*board).square[GET_FILE(cst_start)][GET_RANK(cst_start)] = '\0';
		}

		(*board).extraInfo &= ~a_color;
		(*board).extraInfo |= o_color;

	return TRUE;
}
/* --- END OF MOVE EXECUTION() --- */


static short should_stop = FALSE, should_end = FALSE;

static short foundMove = FALSE; /* if search_best_move terminated without
								   finding a move this is true */

/* these globals are needed by uci_extra */
static short ponderMode = FALSE, infinite = FALSE;
static short searchmoves = FALSE, search_done = FALSE; /* not yet used */
static short command_already_exists = FALSE, thread_is_done = FALSE;

/* these globals will be used by both search_best_move() as well as main() */
static move playedMove = {'\0'};
static move could_Be_Ponder = {'\0'};

static char* uci_extra_buffer = NULL;
static char* uci_extra_word = NULL;

static char* comm = NULL;
static char* wordBuffer = NULL;

static move swap_helper = {'\0','\0', '\0', '\0', '\0', 0};





/* -----------------------------------------
	  SEARCH (GAME TREE) FUNCTIONS
   ----------------------------------------- */
static move* lastMove[MAX_DEPTH+1] = {NULL};

/* These two variations support Third_ALPHA_BETA */
static variation currentVar, queryVar;

short THINK(situation* pos, int your_depth){
	/*curr_depth = your_depth;*/
	char active_color = GET_PIECE_COLOR((*pos).extraInfo);
	short alpha = -32000, beta = 32000, quick_return = 0;
	
	if(your_depth == MAX_DEPTH)
		return evaluate(pos);

	find_moves(pos,&Try_Table[your_depth]);
	quick_return = Try_Table[your_depth].number_of;

	if(!quick_return){
		if(Try_Table[your_depth].mate){
			if(active_color == WHITE)
				return -KING_VALUE+MISTAKE_WINDOW*your_depth;
			else
				return KING_VALUE-MISTAKE_WINDOW*your_depth;
		}
		return STALE_MATE;
	}

	
	whiteNum = blackNum = 0;
	for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){
		if(CurrentBoard->white_Pieces[i] != INVALID_SQUARE)
			++whiteNum;
		if(CurrentBoard->black_Pieces[i] != INVALID_SQUARE)
			++blackNum;
	}

	if(quick_return < 25 && whiteNum + blackNum < 17)
		REAL_DEPTH = MAX_DEPTH + 1;
	else
		REAL_DEPTH = MAX_DEPTH;

	int extra_counter = 0;	/* WHY ? */

	/* A safety copy of the pos-Parameter */
	memcpy(&Try_Board[your_depth],pos,sizeof(situation));	
	
	first_sort(your_depth);	
	
	reset_tree(); /* ROOT = NULL */
	move* currently = NULL;

	/* Pick every possible move */
	for(int i = 0; i < quick_return; ++i){
		currently = Try_Table[your_depth].stored_move+i;
		currently->quality = 0;
		
		currentVar.made[0] = *currently;
		queryVar.made[2] = *currently;
		
		execute_move(&Try_Board[your_depth], currently);
		
		if(memcmp(&Reference[1],&Try_Board[your_depth],sizeof(char)*64)) /* MAGIC NUMBER */
			currently->quality = Second_ALPHA_BETA(&Try_Board[your_depth], your_depth+1, alpha, beta);
		else
			currently->quality = STALE_MATE;

		if(active_color == WHITE && currently->quality > alpha)
			alpha = currently->quality;
		else if(active_color == BLACK & currently->quality < beta)
			beta = currently->quality;

		memcpy(&Try_Board[your_depth],pos,sizeof(situation));

	}
	
	/* selection sort */
	move selection_helper = {'\0'};
	int max_index = 0;
	
	for(int j = 0; j < quick_return; ++j){
			max_index = j;
			for(int i = j+1; i < Try_Table[your_depth].number_of; i++){
				if(active_color == BLACK && Try_Table[your_depth].stored_move[i].quality
					< Try_Table[your_depth].stored_move[max_index].quality)
						max_index = i;
				else if(active_color == WHITE &&
					Try_Table[your_depth].stored_move[i].quality >
					Try_Table[your_depth].stored_move[max_index].quality)
					max_index = i;
			}
			if(j != max_index){
				selection_helper = Try_Table[your_depth].stored_move[j];
				Try_Table[your_depth].stored_move[j] =
					Try_Table[your_depth].stored_move[max_index];
				Try_Table[your_depth].stored_move[max_index] =
					selection_helper;
			}
	}	

	return Try_Table[your_depth].stored_move[0].quality;
}

	

short Second_ALPHA_BETA(situation* pos, int your_depth, short alpha, short beta){
	char active_color = GET_PIECE_COLOR((*pos).extraInfo);
	short quick_return = 0;
	if(your_depth == REAL_DEPTH)
		return evaluate(pos);
	
	find_moves(pos,&Try_Table[your_depth]);

	quick_return = Try_Table[your_depth].number_of;

	if(!quick_return){
		if(Try_Table[your_depth].mate){
			if(active_color == WHITE)
				return -KING_VALUE+MISTAKE_WINDOW*your_depth;
			else
				return KING_VALUE-MISTAKE_WINDOW*your_depth;
		}
		return STALE_MATE;
	}

	/* A safety copy of the pos-Parameter */
	memcpy(&Try_Board[your_depth],pos,sizeof(situation));
	
	hyper_sort(your_depth);
	move* currently = NULL;

	/* Pick every possible move */
	if(active_color == WHITE){
		for(int i = 0; i < quick_return; ++i){
			currently = Try_Table[your_depth].stored_move + i;
			currently->quality = 0;			
			
			currentVar.made[1] = *currently;
			queryVar.made[1] = *currently;			
			
			/* Try_Board = safety copy */
			execute_move(&Try_Board[your_depth], currently);

			currently->quality = 
				Third_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta);				
			if(currently->quality >= beta)
				return beta;
			else if(currently->quality > alpha)
				alpha = currently->quality;

			memcpy(&Try_Board[your_depth],pos,sizeof(situation));
		}
		return alpha;
	}
	else{
		for(int i = 0; i < quick_return; ++i){
			currently = Try_Table[your_depth].stored_move + i;
			currently->quality = 0;

			currentVar.made[1] = *currently;
			queryVar.made[1] = *currently;		
			
			/* Try_Board = safety copy */
			execute_move(&Try_Board[your_depth], currently);

			currently->quality = 
				Third_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta);	
			
			if(currently->quality <= alpha)
				return alpha;
			else if(currently->quality < beta)
				beta = currently->quality;
			memcpy(&Try_Board[your_depth],pos,sizeof(situation));
		}
	}
	return beta;
}



short Third_ALPHA_BETA(situation* pos, int your_depth, short alpha, short beta){
	char active_color = GET_PIECE_COLOR((*pos).extraInfo);
	short quick_return = 0;
	if(your_depth == REAL_DEPTH)
		return evaluate(pos);
	
	find_moves(pos,&Try_Table[your_depth]);

	quick_return = Try_Table[your_depth].number_of;

	if(!quick_return){
		if(Try_Table[your_depth].mate){
			if(active_color == WHITE)
				return -KING_VALUE+MISTAKE_WINDOW*your_depth;
			else
				return KING_VALUE-MISTAKE_WINDOW*your_depth;
		}
		return STALE_MATE;
	}

	/* A safety copy of the pos-Parameter */
	memcpy(&Try_Board[your_depth],pos,sizeof(situation));	
	hyper_sort(your_depth);	/* Oder first_sort */
	
	int tempTruth = FALSE;
	move* currently = NULL;
	short tempQuality = 0; /* Project: Variation containers */

	/* Pick every possible move */
	if(active_color == WHITE){
		for(int i = 0; i < quick_return; ++i){
			currently = Try_Table[your_depth].stored_move+i;
			currently->quality = 0;

			currentVar.made[2] = *currently;
			queryVar.made[0] = *currently;
			currentVar.quality = 0;
			queryVar.quality = 0;
			
			tempTruth = (currentVar.made[2].active_id != currentVar.made[0].active_id);
			
			if(tempTruth){
				tempQuality = get_quality(&queryVar);
				if(tempQuality != NOT_FOUND){
					if(tempQuality >= beta)
						return beta;
					else if(tempQuality > alpha)
						alpha = tempQuality;
				}
				else{
					execute_move(&Try_Board[your_depth], currently);
					currentVar.quality = Standard_ALPHA_BETA(&Try_Board[your_depth],
														your_depth+1, alpha, beta);
					if(currentVar.quality >= beta){
						currentVar.quality = beta;
						insert(&currentVar);
						return beta;
					}
					else if(currentVar.quality > alpha){
						insert(&currentVar);
						alpha = currentVar.quality;
					}
					memcpy(&Try_Board[your_depth],pos,sizeof(situation));
				}
			}
			else{
				/* Try_Board = safety copy */
				execute_move(&Try_Board[your_depth], currently);
				currently->quality = 
					Standard_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta);

				if(currently->quality >= beta)
					return beta;
				else if(currently->quality > alpha)
					alpha = currently->quality;
				memcpy(&Try_Board[your_depth],pos,sizeof(situation));
			}
		}
		return alpha;
	}
	else{
		for(int i = 0; i < quick_return; ++i){
			currently = Try_Table[your_depth].stored_move+i;
			currently->quality = 0;

			currentVar.made[2] = *currently;
			queryVar.made[0] = *currently;
			currentVar.quality = 0;
			queryVar.quality = 0;
			
			tempTruth = (currentVar.made[2].active_id != currentVar.made[0].active_id);
			
			if(tempTruth){
				tempQuality = get_quality(&queryVar);
				if(tempQuality != NOT_FOUND){
					if(tempQuality <= alpha)
						return alpha;
					else if(tempQuality < beta)
						beta = tempQuality;
				}
				else{
					execute_move(&Try_Board[your_depth], currently);
					currentVar.quality = Standard_ALPHA_BETA(&Try_Board[your_depth],
														your_depth+1, alpha, beta);
					
					if(currentVar.quality <= alpha){
						currentVar.quality = alpha;
						insert(&currentVar);
						return alpha;
					}
					else if(currentVar.quality < beta){
						insert(&currentVar);
						beta = currentVar.quality;
					}
					memcpy(&Try_Board[your_depth],pos,sizeof(situation));
				}
			}
			else{
				/* Try_Board = safety copy */
				execute_move(&Try_Board[your_depth], currently);
				currently->quality = 
					Standard_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta);

				if(currently->quality <= alpha)
					return alpha;
				else if(currently->quality < beta)
					beta = currently->quality;
				memcpy(&Try_Board[your_depth],pos,sizeof(situation));
			}
		}
	}
	return beta;
}



short Standard_ALPHA_BETA(situation* pos, int your_depth, short alpha, short beta){
	char active_color = GET_PIECE_COLOR((*pos).extraInfo);
	short quick_return = 0;
	
	if(your_depth == REAL_DEPTH)
		return evaluate(pos);
	
	find_moves(pos,&Try_Table[your_depth]);

	quick_return = Try_Table[your_depth].number_of;

	if(!quick_return){
		if(Try_Table[your_depth].mate){
			if(active_color == WHITE)
				return -KING_VALUE+MISTAKE_WINDOW*your_depth;
			else
				return KING_VALUE-MISTAKE_WINDOW*your_depth;
		}
		return STALE_MATE;
	}

	/* A safety copy of the pos-Parameter */
	memcpy(&Try_Board[your_depth],pos,sizeof(situation));
	
	hyper_sort(your_depth);

	/* Pick every possible move */
	move* currently = NULL;
	if(active_color == WHITE){
		for(int i = 0; i < quick_return; ++i){
			currently = &(Try_Table[your_depth].stored_move[i]);
			/* Try_Board = safety copy */
			execute_move(&Try_Board[your_depth], currently);

			(*currently).quality = ((your_depth+1 > BUCKET_DEPTH) ?
				Mediocre_ALPHA_BETA(&Try_Board[your_depth], your_depth+1, alpha, beta)
				: Standard_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta));		
			
			if((*currently).quality >= beta)
				return beta;
			else if((*currently).quality > alpha)
				alpha =(*currently).quality;
			memcpy(&Try_Board[your_depth],pos,sizeof(situation));
		}
		return alpha;
	}
	else{
		for(int i = 0; i < quick_return; ++i){
			currently = &(Try_Table[your_depth].stored_move[i]);
			/* Try_Board = safety copy */
			execute_move(&Try_Board[your_depth], currently);
			
			(*currently).quality = ((your_depth+1 > BUCKET_DEPTH) ?
				Mediocre_ALPHA_BETA(&Try_Board[your_depth], your_depth+1, alpha, beta)
				: Standard_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta));
			
			if((*currently).quality <= alpha)
				return alpha;
			else if((*currently).quality < beta)
				beta = (*currently).quality;
			memcpy(&Try_Board[your_depth],pos,sizeof(situation));
		}
	}
	return beta;
}





/* This function is called at depth = 5 */
short Mediocre_ALPHA_BETA(situation* pos, int your_depth, short alpha, short beta){
	
	char active_color = GET_PIECE_COLOR((*pos).extraInfo);
	short quick_return = 0;
	
	if(your_depth == REAL_DEPTH)
		return evaluate(pos);
	
	find_moves(pos,&Try_Table[your_depth]);

	quick_return = Try_Table[your_depth].number_of;

	if(!quick_return){
		if(Try_Table[your_depth].mate){
			if(active_color == WHITE)
				return -KING_VALUE+MISTAKE_WINDOW*your_depth;
			else
				return KING_VALUE-MISTAKE_WINDOW*your_depth;
		}
		return STALE_MATE;
	}

	/* A safety copy of the pos-Parameter */
	memcpy(&Try_Board[your_depth],pos,sizeof(situation));
	
	mediocre_sort(your_depth);

	/* Pick every possible move */
	move* currently = NULL;
	if(active_color == WHITE){
		for(int i = 0; i < quick_return; ++i){
			currently = &(Try_Table[your_depth].stored_move[i]);
			/* Try_Board = safety copy */
			execute_move(&Try_Board[your_depth], currently);

			(*currently).quality = 
				Bucket_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta);		
			
			if((*currently).quality >= beta)
				return beta;
			else if((*currently).quality > alpha)
				alpha =(*currently).quality;
			memcpy(&Try_Board[your_depth],pos,sizeof(situation));
		}
		return alpha;
	}
	else{
		for(int i = 0; i < quick_return; ++i){
			currently = &(Try_Table[your_depth].stored_move[i]);
			/* Try_Board = safety copy */
			execute_move(&Try_Board[your_depth], currently);
			
			(*currently).quality =
				Bucket_ALPHA_BETA(&Try_Board[your_depth], your_depth+1, alpha, beta);
			
			if((*currently).quality <= alpha)
				return alpha;
			else if((*currently).quality < beta)
				beta = (*currently).quality;
			memcpy(&Try_Board[your_depth],pos,sizeof(situation));
		}
	}
	return beta;
}

/*	This version of ALPHA_BETA is called, when your_depth >= REAL_DEPTH-1. */
short Bucket_ALPHA_BETA(situation* pos, int your_depth, short alpha, short beta){
	
	char active_color = GET_PIECE_COLOR((*pos).extraInfo);
	short quick_return = 0;
	
	/* QUIESCENT PROBLEM. That's why we need to find out the priority of
	   the last move, which is a function of buck_i[your_depth-1] */
	if(your_depth == REAL_DEPTH){
		short horizon_penalty = 0, result = 0;	/* Possibly we need a Penalty, if the last
												   move was a capturing move */
		if(buck_i[your_depth-1] < MEDIOCRE_MOVE){
			move* lastMove = 
				depth_pre_sort[your_depth-1][buck_i[your_depth-1]].stored_move+buck_j[your_depth-1];
			if(quick_check(pos-1, lastMove->dest)){
				unsigned active_piece_type = 
					GET_PIECE_TYPE((pos-1)->square[GET_FILE(lastMove->start)][GET_RANK(lastMove->start)]);
				horizon_penalty = glob_piece_values[active_piece_type];
				if(active_color == WHITE)
					horizon_penalty = -horizon_penalty;
			}

		}

		/* A different evaluation function for different 
		   parts of the game */
		whiteNum = blackNum = 0;
		for(int i = 0; i < MAXIMUM_NUMBER_OF_PIECES; ++i){
			if(pos->white_Pieces[i] != INVALID_SQUARE)
				++whiteNum;
			if(pos->black_Pieces[i] != INVALID_SQUARE)
				++blackNum;
		}

		if(whiteNum + blackNum > OPENING) 
			result = evaluate_opening(pos);
		else
			result = evaluate(pos);
		
		if(abs(result) < 14000)
			return result-horizon_penalty;
		else
			return result;
	}
	find_moves(pos,&Try_Table[your_depth]);

	Try_Table[your_depth].number_of;	/* NEW -- */

	if(!Try_Table[your_depth].number_of){
		if(Try_Table[your_depth].mate){
			if(active_color == WHITE)
				return -KING_VALUE+MISTAKE_WINDOW*your_depth;
			else
				return KING_VALUE-MISTAKE_WINDOW*your_depth;
		}
		return STALE_MATE;
	}

	/* A safety copy of the pos-Parameter */
	memcpy(&Try_Board[your_depth],pos,sizeof(situation));
	
	/* Pick every possible move */
	move* currently = NULL;

	/* #### Read the moves directly from the buckets instead of re-copying them
			to the Try_Table. Due to buck_i and buck_j the final evaluation
			function should be able to determine whether a horizon penalty
			is necessary. ###### */

	buck_sort(your_depth);
	if(active_color == WHITE){
		for(buck_i[your_depth] = 0; 
			buck_i[your_depth] < MEDIOCRE_CRITERIA; 
			++buck_i[your_depth]){
			quick_return = depth_pre_sort[your_depth][buck_i[your_depth]].number_of;
			for(buck_j[your_depth] = 0; 
				buck_j[your_depth] < quick_return; 
				++buck_j[your_depth]){
				currently =
					&(depth_pre_sort[your_depth][buck_i[your_depth]].stored_move[buck_j[your_depth]]);
					/* Try_Board = safety copy */
					execute_move(&Try_Board[your_depth], currently);
					(*currently).quality = Bucket_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta);		
					if((*currently).quality >= beta)
						return beta;
					else if((*currently).quality > alpha)
						alpha =(*currently).quality;
					memcpy(&Try_Board[your_depth],pos,sizeof(situation));
			}
		}
		return alpha;
	}	

	for(buck_i[your_depth] = 0; 
		buck_i[your_depth] < MEDIOCRE_CRITERIA; 
		++buck_i[your_depth]){
			quick_return = depth_pre_sort[your_depth][buck_i[your_depth]].number_of;
			for(buck_j[your_depth] = 0; 
				buck_j[your_depth] < quick_return; 
				++buck_j[your_depth]){
					currently =
						&(depth_pre_sort[your_depth][buck_i[your_depth]].stored_move[buck_j[your_depth]]);
					/* Try_Board = safety copy */
					execute_move(&Try_Board[your_depth], currently);
					(*currently).quality = Bucket_ALPHA_BETA(&Try_Board[your_depth],your_depth+1, alpha, beta);		
					if((*currently).quality <= alpha)
						return alpha;
					else if((*currently).quality < beta)
						beta =(*currently).quality;
					memcpy(&Try_Board[your_depth],pos,sizeof(situation));
			}
	}	
	return beta;
}

int search_best_move(char* result){
	search_done = FALSE;
	foundMove = FALSE;
	char move_buffer[6] = {'\0'};
	
	short move_index = -1;

	if(memcmp(&GlobalNullMove,&could_Be_Ponder, sizeof(move)) != 0){
		execute_move(CurrentBoard,&could_Be_Ponder);
		memmove(Reference,Reference+1,4*sizeof(situation));
		memcpy(Reference+4,CurrentBoard,sizeof(situation));
	}
		
	THINK(CurrentBoard,0);

		if(move_index == -1 &&
			Try_Table[0].number_of){
			move_index = 0;
			move_to_alg(&Try_Table[0].stored_move[0],move_buffer);
		}

		else if(move_index == -1 &&
			!(Try_Table[0].number_of)){
			foundMove = FALSE;
			strcpy(result, "0000\n");
			search_done = TRUE;
			return TRUE;
			}

		else if(move_index != -1){
			Try_Table[0].stored_move[0].quality = Try_Table[0].stored_move[move_index].quality;
		}
		
	move_to_alg(&(Try_Table[0].stored_move[move_index]), result);

	foundMove = TRUE;
	
	search_done = TRUE;

	return search_done;
}


/* MAIN() == UCI COMMUNICATION */
int main(){
	char helpMoveBuffer[6] = {'\0'};
	
	srand(time(NULL));
	
	setbuf(stdout, NULL);
	setbuf(stdin, NULL);
	
	/*
	logFile = fopen("suffLog","w"); 
	if(!logFile)
		return 0;*/
	
	
	int endIt = FALSE;
	
	
	int loop_i = 0, loop_j = 0;

	comm = (char*) calloc(2048,sizeof(char));
	wordBuffer = (char*) calloc(256,sizeof(char));

	if(!comm || !wordBuffer)
		return 0;
	

	char move_buffer[8] = {'\0'};
	
	short no_Position = TRUE;
	short innerLoop = TRUE;
	short innerHelper = 0;
	short serious_problem = FALSE;
	short pre_Moves = FALSE;
	
	/* main loop */
	do{
	loop_i = loop_j = 0;
	innerLoop = TRUE;
	innerHelper = -1;
	if(should_end)
		break;

	if(!command_already_exists){

		if(fgets(comm,2048,stdin) == 0){
			endIt = TRUE;
			continue;
		}
	}
	else
		command_already_exists = FALSE;

	while(comm[loop_i] == ' ' || comm[loop_i] == '\t')
		loop_i++;

	while(comm[loop_i] != ' ' && comm[loop_i] != '\t'
		&& comm[loop_i] != '\n'){
		wordBuffer[loop_j] = comm[loop_i];
		++loop_i; ++loop_j;
	}
		
		/* command with no additional parameters */
		if(comm[loop_i] == '\n'){
			
			wordBuffer[loop_j] = '\0';

			if(strncmp(wordBuffer, "uci", 3*sizeof(char)) == 0){
				fprintf(stdout,"id name SUFF 0.3a JA\n");
				fprintf(stdout,"id author Clemens Pruell\n");
				fprintf(stdout,"uciok\n");
				loop_i = loop_j = 0;
			continue;
			}

			else if(strncmp(wordBuffer, "isready",7*sizeof(char)) == 0){
				if(!fixed_Destination){
					if(!init())
						endIt = TRUE;
				}
			fprintf(stdout,"readyok\n");
			loop_i = loop_j = 0;
			continue;
			}

			else if(strncmp(wordBuffer, "quit", 4*sizeof(char)) == 0){
				endIt = TRUE;
				continue;
			}
				
			else if(strncmp(wordBuffer, "ponderhit",9*sizeof(char)) == 0){
				if(!CurrentBoard)
					break;
				ponderMode = FALSE;
				search_best_move(move_buffer);
				fprintf(stdout,"bestmove %s\n", move_buffer);
			}

			else{
				loop_i = loop_j = 0;
				continue;
			}
		}

		
		/* this means we got a command with additional parameters */
		else if(comm[loop_i] == ' ' || comm[loop_i] == '\t'){


			wordBuffer[loop_j] = '\0';
			if(strncmp(wordBuffer, "position",8*sizeof(char)) == 0){
				ponderMode = infinite = searchmoves = FALSE;
				innerHelper = -1;
				/* this could possibly be position */
				while(comm[loop_i] == ' ' || comm[loop_i] == '\t')
					++loop_i;
				loop_j = 0;

				while(comm[loop_i] != ' ' && comm[loop_i] != '\t'
					&& comm[loop_i] != '\n'){
						wordBuffer[loop_j] = comm[loop_i];
						++loop_j; ++loop_i;
					}
					wordBuffer[loop_j] = '\0';

					if(strncmp(wordBuffer,"startpos",8*sizeof(char)) == 0){						
						memset(&could_Be_Ponder,'\0',sizeof(move));
						memset(&playedMove, '\0', sizeof(move));				
						
						if(!create_internal_board(StartPosition)){
							endIt = TRUE;
							continue;
						}
						else{
							memset(Reference, '\0', NUMBER_OF_REFERENCES*sizeof(situation));
							memcpy(Reference+4, CurrentBoard,sizeof(situation));
						}

						if(comm[loop_i] == '\n'){
							no_Position = FALSE;
							continue;
						}
					}

					else if(strncmp(wordBuffer, "fen",3*sizeof(char)) == 0){
						memset(&could_Be_Ponder,'\0',sizeof(move));
						memset(&playedMove, '\0', sizeof(move));
						loop_j = 0;
						while(comm[loop_i] == ' ' || comm[loop_i] == '\t')
							++loop_i;
						for(int i = 0; i < 6;){
							wordBuffer[loop_j] = comm[loop_i];
							if(comm[loop_i] == ' ' || comm[loop_i] == '\t'
								|| comm[loop_i] == '\n'){
								++i;
								if(comm[loop_i] == '\n' && i != 6){
									++loop_i;
									break;
								}
								}
							if(i == 6)
								wordBuffer[loop_j] = '\0';
							++loop_j; ++loop_i;
							if(loop_j > 254)
								break;
						}
						if(!create_internal_board(wordBuffer)){
							endIt = TRUE;
							continue;
						}
						else{
							memset(Reference, '\0', NUMBER_OF_REFERENCES*sizeof(situation));
							memcpy(Reference+4,CurrentBoard,sizeof(situation));
						}

						if(comm[loop_i-1] == '\n'){
							no_Position = FALSE;
							continue;
						}
					}
					while(comm[loop_i] == ' ' || comm[loop_i] == '\t')
						loop_i++;
					loop_j = 0;

					while(comm[loop_i] != ' ' && comm[loop_i] != '\t'
						&& comm[loop_i] != '\n'){
							wordBuffer[loop_j] = comm[loop_i];
							++loop_j; ++ loop_i;
						}
						wordBuffer[loop_j] = '\0';

						if(strncmp(wordBuffer,"moves",5*sizeof(char)) == 0)
							pre_Moves = TRUE;						

						if(comm[loop_i] == '\n')
							continue;

						while(comm[loop_i] == ' ' || comm[loop_i] == '\t')
							++loop_i;
						if(!CurrentBoard){
							endIt = TRUE;
							continue;
						}
						if(comm[loop_i] == '\n')
							continue;
						do{
						/* parse - find - execute */
						loop_j = 0;
						move_buffer[4] = '\0'; /* promote */
						while(comm[loop_i] != ' ' && comm[loop_i] != '\t'
							&& comm[loop_i] != '\n' && loop_j < 6){							
							move_buffer[loop_j] = comm[loop_i];
							++loop_i; ++loop_j;
							}
						move_buffer[5] = '\0';						
						
						if(move_buffer[0] < 'a' || move_buffer[0] > 'h'
							|| move_buffer[1] < '1' || move_buffer[1] > '8')
							
							{
								no_Position = FALSE;
								break;							
							}

						if(innerHelper != -1){
							execute_move(CurrentBoard,&could_Be_Ponder);
							memmove(Reference, Reference+1, 4*sizeof(situation));
							memcpy(Reference+4, CurrentBoard, sizeof(situation));
							memset(&could_Be_Ponder, '\0', sizeof(move));
						}
						
						
						alg_to_move(move_buffer,&playedMove);
						find_moves(CurrentBoard,Possible_Moves);

						innerHelper = get_Move_index(&playedMove,Possible_Moves);
						

						if(innerHelper == -1){
							innerLoop = FALSE;
							continue;
						}
						else
							could_Be_Ponder = (*Possible_Moves).stored_move[innerHelper];
						
							
						if(comm[loop_i] == '\n'){
							no_Position = FALSE;
							break;
						}

						while(comm[loop_i] == ' ' || comm[loop_i] == '\t')
							loop_i++;

							

						}while(innerLoop);


						if(innerHelper == -1){
							endIt = TRUE;
							continue;
						}
					} /* End of position_if */


					/* here comes the else if 'go' */
					else if(strcmp(wordBuffer,"go") == 0){
						if(no_Position)
							continue;

						do{
						
						loop_j = 0;

						while(comm[loop_i] == ' ' || comm[loop_i] == '\t')
							++loop_i;

						while(comm[loop_i] != ' ' && comm[loop_i] != '\t'
							&& comm[loop_i] != '\n'){
							wordBuffer[loop_j] = comm[loop_i];
							++loop_j; ++loop_i;
							}

							wordBuffer[loop_j] = '\0';
							if(strncmp(wordBuffer,"ponder",6*sizeof(char)) == 0)
								ponderMode = TRUE;
/*
							else if(strncmp(wordBuffer, "infinite",8*sizeof(char)) == 0)
								infinite = TRUE;								

							else if(strncmp(wordBuffer, "searchmoves", 11* sizeof(char)) == 0)
								searchmoves = TRUE;*/
							
							if(comm[loop_i] == '\n' && ponderMode == FALSE){
								should_stop = FALSE;
								
								search_best_move(move_buffer); 
								
								pre_Moves = FALSE;

								short color_helper = GET_PIECE_COLOR(CurrentBoard->extraInfo);
								if(Try_Table[0].number_of){
									color_helper = ((color_helper == BLACK) ?
										-Try_Table[0].stored_move[0].quality :
										 Try_Table[0].stored_move[0].quality);
									fprintf(stdout,"info depth %d score cp %d pv %s\n",
									REAL_DEPTH,
									color_helper,
									move_buffer);
								}
								
								fprintf(stdout,"bestmove %s\n",move_buffer);
																
								should_stop = FALSE;

								break;
							}
						}while(TRUE);

						no_Position = TRUE;

					} /* end of 'go' */
				

		} /* end of else if with multiple parameters */		

	if(comm[loop_i] != '\n' )
		return 0;


}while(endIt != TRUE && serious_problem != TRUE && !should_end); /* End of Main Loop */

	return 0;
}

/* ---- ---- ---- END OF PROGRAM ---- ---- ---- */
